@@ -1,12 +1,10 @@
-// $Id: brkpt.cpp,v 1.30.2.2 2004/05/19 20:35:51 jsixt Exp $
+// $Id: brkpt.cpp,v 1.38 2004/05/20 15:08:42 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
 
 #include <kapp.h>			/* i18n */
-#if QT_VERSION >= 200
 #include <klocale.h>			/* i18n */
-#endif
 #include <kiconloader.h>
 #include <ksimpleconfig.h>
 #include <qdialog.h>
@@ -17,7 +15,6 @@
 #include "debugger.h"
 #include "brkpt.h"
 #include "dbgdriver.h"
-#include "brkpt.moc"
 #include <ctype.h>
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -168,10 +165,12 @@ void BreakpointTable::addBP()
     // set a breakpoint at the specified text
     QString bpText = m_bpEdit.text();
     bpText = bpText.stripWhiteSpace();
-    if (m_debugger->isReady()) {
-	m_debugger->driver()->executeCmd(DCbreaktext, bpText);
-	// clear text if successfully set
-	m_bpEdit.setText("");
+    if (m_debugger->isReady())
+    {
+	Breakpoint* bp = new Breakpoint;
+	bp->text = bpText;
+
+	m_debugger->setBreakpoint(bp, false);
     }
 }
 
@@ -181,7 +180,11 @@ void BreakpointTable::addWP()
     QString wpExpr = m_bpEdit.text();
     wpExpr = wpExpr.stripWhiteSpace();
     if (m_debugger->isReady()) {
-	m_debugger->driver()->executeCmd(DCwatchpoint, wpExpr);
+	Breakpoint* bp = new Breakpoint;
+	bp->type = Breakpoint::watchpoint;
+	bp->text = wpExpr;
+
+	m_debugger->setBreakpoint(bp, false);
     }
 }
 
@@ -191,7 +194,12 @@ void BreakpointTable::removeBP()
     if (bp == 0)
 	return;
 
-    m_debugger->driver()->executeCmd(DCdelete, bp->id);
+    Breakpoint* brk = m_debugger->breakpointById(bp->id);
+    if (brk != 0) {
+	m_debugger->deleteBreakpoint(brk);
+	// note that both brk and bp may be deleted by now
+	// (if bp was an orphaned breakpoint)
+    }
 }
 
 void BreakpointTable::enadisBP()
@@ -200,8 +208,10 @@ void BreakpointTable::enadisBP()
     if (bp == 0)
 	return;
 
-    DbgCommand cmd = bp->enabled() ? DCdisable : DCenable;
-    m_debugger->driver()->executeCmd(cmd, bp->id);
+    Breakpoint* brk = m_debugger->breakpointById(bp->id);
+    if (brk != 0) {
+	m_debugger->enableDisableBreakpoint(brk);
+    }
 }
 
 void BreakpointTable::viewBP()
@@ -236,25 +246,21 @@ void BreakpointTable::updateUI()
     m_btConditional.setEnabled(enableChkpt);
 }
 
-#if QT_VERSION < 200
-# define MOUSEPRESS Event_MouseButtonPress
-#else
-# define MOUSEPRESS QEvent::MouseButtonPress
-#endif
-
 bool BreakpointTable::eventFilter(QObject* ob, QEvent* ev)
 {
-    if (ev->type() == MOUSEPRESS)
+    if (ev->type() == QEvent::MouseButtonPress)
     {
 	QMouseEvent* mev = static_cast<QMouseEvent*>(ev);
 	if (mev->button() == MidButton) {
 	    // enable or disable the clicked-on item
 	    BreakpointItem* bp =
 		static_cast<BreakpointItem*>(m_list.itemAt(mev->pos()));
-	    if (bp != 0 && m_debugger->canChangeBreakpoints())
+	    if (bp != 0)
 	    {
-		DbgCommand cmd = bp->enabled() ? DCdisable : DCenable;
-		m_debugger->driver()->executeCmd(cmd, bp->id);
+		Breakpoint* brk = m_debugger->breakpointById(bp->id);
+		if (brk != 0) {
+		    m_debugger->enableDisableBreakpoint(brk);
+		}
 	    }
 	    return true;
 	}
@@ -315,39 +321,10 @@ void BreakpointTable::updateBreakpointCondition(int id,
 						const QString& condition,
 						int ignoreCount)
 {
-    BreakpointItem* bp = itemByBreakId(id);
-    if (bp == 0)
-	return;				/* breakpoint no longer exists */
-
-    bool changed = false;
-
-    if (bp->condition != condition) {
-	// change condition
-	m_debugger->driver()->executeCmd(DCcondition, condition, bp->id);
-	bp->condition = condition;
-	changed = true;
+    Breakpoint* brk = m_debugger->breakpointById(id);
+    if (brk != 0) {
+	m_debugger->conditionalBreakpoint(brk, condition, ignoreCount);
     }
-    if (bp->ignoreCount != ignoreCount) {
-	// change ignore count
-	m_debugger->driver()->executeCmd(DCignore, bp->id, ignoreCount);
-	changed = true;
-    }
-    if (changed) {
-	// get the changes
-	m_debugger->driver()->queueCmd(DCinfobreak, DebuggerDriver::QMoverride);
-    }
-}
-
-
-BreakpointItem* BreakpointTable::itemByBreakId(int id)
-{
-    for (QListViewItem* it = m_list.firstChild(); it != 0; it = it->nextSibling()) {
-	BreakpointItem* bp = static_cast<BreakpointItem*>(it);
-	if (bp->id == id) {
-	    return bp;
-	}
-    }
-    return 0;
 }
 
 
@@ -364,31 +341,24 @@ void BreakpointTable::initListAndIcons()
     m_list.setSorting(-1);
 
     // add pixmaps
-#if QT_VERSION < 200
-    KIconLoader* loader = kapp->getIconLoader();
-    QPixmap brkena = loader->loadIcon("brkena.xpm");
-    QPixmap brkdis = loader->loadIcon("brkdis.xpm");
-    QPixmap watchena = loader->loadIcon("watchena.xpm");
-    QPixmap watchdis = loader->loadIcon("watchdis.xpm");
-    QPixmap brktmp = loader->loadIcon("brktmp.xpm");
-    QPixmap brkcond = loader->loadIcon("brkcond.xpm");
-#else
     QPixmap brkena = UserIcon("brkena.xpm");
     QPixmap brkdis = UserIcon("brkdis.xpm");
     QPixmap watchena = UserIcon("watchena.xpm");
     QPixmap watchdis = UserIcon("watchdis.xpm");
     QPixmap brktmp = UserIcon("brktmp.xpm");
     QPixmap brkcond = UserIcon("brkcond.xpm");
-#endif
+    QPixmap brkorph = UserIcon("brkorph.xpm");
+
     /*
-     * There are 16 different pixmaps: The basic enabled or disabled
+     * There are 32 different pixmaps: The basic enabled or disabled
      * breakpoint, plus an optional overlaid brktmp icon plus an optional
-     * overlaid brkcond icon. Then the same sequence for watchpoints.
+     * overlaid brkcond icon, plus an optional overlaid brkorph icon. Then
+     * the same sequence for watchpoints.
      */
-    m_icons.setSize(16);
+    m_icons.setSize(32);
     QPixmap canvas(16,16);
 
-    for (int i = 0; i < 16; i++) {
+    for (int i = 0; i < 32; i++) {
 	{
 	    QPainter p(&canvas);
 	    // clear canvas
@@ -407,6 +377,10 @@ void BreakpointTable::initListAndIcons()
 	    if (i & 4) {
 		p.drawPixmap(1,1, brkcond);
 	    }
+	    // orphan overlay
+	    if (i & 16) {
+		p.drawPixmap(1,1, brkorph);
+	    }
 	}
 	canvas.setMask(canvas.createHeuristicMask());
 	m_icons[i] = canvas;
@@ -425,10 +399,28 @@ void BreakpointItem::display()
 	code += 4;
     if (type == watchpoint)
 	code += 8;
+    if (isOrphaned())
+	code += 16;
     setPixmap(0, lb->m_icons[code]);
 
     // more breakpoint info
-    setText(0, location);
+    if (!location.isEmpty()) {
+	setText(0, location);
+    } else if (!Breakpoint::text.isEmpty()) {
+	setText(0, Breakpoint::text);
+    } else if (!fileName.isEmpty()) {
+	// use only the file name portion
+	QString file = fileName;
+	int slash = file.findRev('/');
+	if (slash >= 0) {
+	    file = file.mid(slash+1);
+	}
+	// correct zero-based line-numbers
+	setText(0, file + ":" + QString::number(lineNo+1));
+    } else {
+	setText(0, "*" + address.asString());
+    }
+
     int c = 0;
     setText(++c, address.asString());
     QString tmp;
@@ -464,7 +456,7 @@ ConditionalDlg::ConditionalDlg(QWidget* parent) :
 	m_inputs(2, 2, 10),
 	m_buttons(4)
 {
-    QString title = kapp->getCaption();
+    QString title = kapp->caption();
     title += i18n(": Conditional breakpoint");
     setCaption(title);
 
@@ -533,3 +525,6 @@ void ConditionalDlg::setIgnoreCount(uint count)
     }
     m_ignoreCount.setText(text);
 }
+
+
+#include "brkpt.moc"
