@@ -1,4 +1,4 @@
-// $Id: dbgdriver.h,v 1.16.2.3 2003/09/07 18:36:22 jsixt Exp $
+// $Id: dbgdriver.h,v 1.33 2004/08/09 20:15:22 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -9,6 +9,7 @@
 #include <qptrqueue.h>
 #include <qptrlist.h>
 #include <qfile.h>
+#include <qregexp.h>
 #include <kprocess.h>
 
 
@@ -16,6 +17,7 @@ class VarTree;
 class ExprWnd;
 class KDebugger;
 class QStrList;
+class QStringList;
 
 
 /**
@@ -55,6 +57,7 @@ enum DbgCommand {
 	DCsetargs,
 	DCsetenv,
 	DCunsetenv,
+	DCsetoption,                    /* debugger options */
 	DCcd,
 	DCbt,
 	DCrun,
@@ -76,6 +79,7 @@ enum DbgCommand {
 	DCenable,
 	DCdisable,
 	DCprint,
+	DCprintDeref,
 	DCprintStruct,
 	DCprintQStringStruct,
 	DCframe,
@@ -85,7 +89,9 @@ enum DbgCommand {
 	DCinfothreads,
 	DCinfobreak,
 	DCcondition,
-	DCignore
+	DCsetpc,
+	DCignore,
+	DCsetvariable
 };
 
 enum RunDevNull {
@@ -120,6 +126,8 @@ enum MemoryDumpType {
     MDTformatmask = 0xf0
 };
 
+struct Breakpoint;
+
 /**
  * Debugger commands are placed in a queue. Only one command at a time is
  * sent down to the debugger. All other commands in the queue are retained
@@ -139,6 +147,8 @@ struct CmdQueueItem
     // remember file position
     QString m_fileName;
     int m_lineNo;
+    // the breakpoint info
+    Breakpoint* m_brkpt;
     // whether command was emitted due to direct user request (only set when relevant)
     bool m_byUser;
 
@@ -149,6 +159,7 @@ struct CmdQueueItem
 	m_expr(0),
 	m_exprWnd(0),
 	m_lineNo(0),
+	m_brkpt(0),
 	m_byUser(false)
     { }
 };
@@ -166,6 +177,7 @@ struct Breakpoint
     bool temporary;
     bool enabled;
     QString location;
+    QString text;			/* text if set using DCbreaktext */
     DbgAddr address;			/* exact address of breakpoint */
     QString condition;			/* condition as printed by gdb */
     int ignoreCount;			/* ignore next that may hits */
@@ -173,6 +185,8 @@ struct Breakpoint
     // the following items repeat the location, but in a better usable way
     QString fileName;
     int lineNo;				/* zero-based line number */
+    Breakpoint();
+    bool isOrphaned() const { return id < 0; }
 };
 
 /**
@@ -215,6 +229,7 @@ struct RegisterInfo
     QString regName;
     QString rawValue;
     QString cookedValue;		/* may be empty */
+    QString type;			/* of vector register if not empty */
 };
 
 /**
@@ -255,6 +270,11 @@ public:
      */
     virtual QString defaultInvocation() const = 0;
 
+    /**
+     * Returns a list of options that can be  turned on and off.
+     */
+    virtual QStringList boolOptionList() const = 0;
+
     virtual bool startup(QString cmdStr);
     void dequeueCmdByVar(VarTree* var);
     void setLogFileName(const QString& fname) { m_logFileName = fname; }
@@ -281,13 +301,9 @@ public:
 
 protected:
     char* m_output;			/* normal gdb output */
-    int m_outputLen;			/* current accumulated output */
-    int m_outputAlloc;			/* space available in m_gdbOutput */
-#if QT_VERSION < 200
-    typedef QString DelayedStr;
-#else
+    size_t m_outputLen;			/* amount of data so far accumulated in m_output */
+    size_t m_outputAlloc;		/* space available in m_output */
     typedef QCString DelayedStr;
-#endif
     QQueue<DelayedStr> m_delayedOutput;	/* output colleced while we have receivedOutput */
 					/* but before signal wroteStdin arrived */
 
@@ -353,6 +369,11 @@ public:
      */
     virtual void interruptInferior() = 0;
 
+    /**
+     * Specifies the command that prints the QString data.
+     */
+    virtual void setPrintQStringDataCmd(const char* cmd) = 0;
+
     /**
      * Parses the output as an array of QChars.
      */
@@ -401,11 +422,12 @@ public:
      * @param output The output of the debugger.
      * @param id Returns the breakpoint id.
      * @param file Returns the file name in which the breakpoint is.
-     * @param lineNo Returns he zero-based line number of the breakpoint.
+     * @param lineNo Returns the zero-based line number of the breakpoint.
+     * @param address Returns the address of the breakpoint.
      * @return False if there was no breakpoint.
      */
     virtual bool parseBreakpoint(const char* output, int& id,
-				 QString& file, int& lineNo) = 0;
+				 QString& file, int& lineNo, QString& address) = 0;
 
     /**
      * Parses the output of the DCinfolocals command.
@@ -495,6 +517,18 @@ public:
      */
     virtual QString parseMemoryDump(const char* output, QList<MemoryDump>& memdump) = 0;
 
+    /**
+     * Parses the output of the DCsetvariable command. Returns an empty
+     * string if no error was found; otherwise it contains an error
+     * message.
+     */
+    virtual QString parseSetVariable(const char* output) = 0;
+
+    /**
+     * Returns a value that the user can edit.
+     */
+    virtual QString editableValue(VarTree* value);
+
 protected:
     /** Removes all commands from the low-priority queue. */
     void flushLoPriQueue();
@@ -527,17 +561,18 @@ protected:
     virtual int commSetupDoneC();
 
     char m_prompt[10];
-    int m_promptLen;
+    size_t m_promptMinLen;
     char m_promptLastChar;
+    QRegExp m_promptRE;
 
     // log file
     QString m_logFileName;
     QFile m_logFile;
 
 protected slots:
-    void slotReceiveOutput(KProcess*, char* buffer, int buflen);
-    void slotCommandRead(KProcess*);
-    void slotExited(KProcess*);
+    virtual void slotReceiveOutput(KProcess*, char* buffer, int buflen);
+    virtual void slotCommandRead(KProcess*);
+    virtual void slotExited(KProcess*);
     
 signals:
     /**
