@@ -1,4 +1,4 @@
-// $Id: debugger.cpp,v 1.106.2.5 2003/08/16 16:22:28 jsixt Exp $
+// $Id: debugger.cpp,v 1.142 2005/01/01 21:23:31 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -14,14 +14,11 @@
 #include <qregexp.h>
 #include <qfileinfo.h>
 #include <qlistbox.h>
+#include <qstringlist.h>
 #include <kapp.h>
 #include <kconfig.h>
-#if QT_VERSION >= 200
 #include <klocale.h>			/* i18n */
 #include <kmessagebox.h>
-#else
-#include <kmsgbox.h>
-#endif
 #include <ctype.h>
 #include <stdlib.h>			/* strtol, atoi */
 #ifdef HAVE_UNISTD_H
@@ -33,9 +30,7 @@
 KDebugger::KDebugger(QWidget* parent,
 		     ExprWnd* localVars,
 		     ExprWnd* watchVars,
-		     QListBox* backtrace,
-		     DebuggerDriver* driver
-		     ) :
+		     QListBox* backtrace) :
 	QObject(parent, "debugger"),
 	m_ttyLevel(ttyFull),
 	m_memoryFormat(MDTword | MDThex),
@@ -45,39 +40,25 @@ KDebugger::KDebugger(QWidget* parent,
 	m_sharedLibsListed(false),
 	m_typeTable(0),
 	m_programConfig(0),
-	m_d(driver),
+	m_d(0),
 	m_localVariables(*localVars),
 	m_watchVariables(*watchVars),
-	m_btWindow(*backtrace),
-	m_animationTimer(this),
-	m_animationInterval(0)
+	m_btWindow(*backtrace)
 {
     m_envVars.setAutoDelete(true);
+    m_brkpts.setAutoDelete(true);
 
     connect(&m_localVariables, SIGNAL(expanding(KTreeViewItem*,bool&)),
 	    SLOT(slotLocalsExpanding(KTreeViewItem*,bool&)));
     connect(&m_watchVariables, SIGNAL(expanding(KTreeViewItem*,bool&)),
 	    SLOT(slotWatchExpanding(KTreeViewItem*,bool&)));
+    connect(&m_localVariables, SIGNAL(editValueCommitted(int, const QString&)),
+	    SLOT(slotValueEdited(int, const QString&)));
+    connect(&m_watchVariables, SIGNAL(editValueCommitted(int, const QString&)),
+	    SLOT(slotValueEdited(int, const QString&)));
 
     connect(&m_btWindow, SIGNAL(highlighted(int)), SLOT(gotoFrame(int)));
 
-    connect(m_d, SIGNAL(activateFileLine(const QString&,int,const DbgAddr&)),
-	    this, SIGNAL(activateFileLine(const QString&,int,const DbgAddr&)));
-
-    // debugger process
-    connect(m_d, SIGNAL(processExited(KProcess*)), SLOT(gdbExited(KProcess*)));
-    connect(m_d, SIGNAL(commandReceived(CmdQueueItem*,const char*)),
-	    SLOT(parse(CmdQueueItem*,const char*)));
-    connect(m_d, SIGNAL(wroteStdin(KProcess*)), SIGNAL(updateUI()));
-    connect(m_d, SIGNAL(inferiorRunning()), SLOT(slotInferiorRunning()));
-    connect(m_d, SIGNAL(enterIdleState()), SLOT(backgroundUpdate()));
-    connect(m_d, SIGNAL(enterIdleState()), SIGNAL(updateUI()));
-
-    // animation
-    connect(&m_animationTimer, SIGNAL(timeout()), SIGNAL(animationTimeout()));
-    // special update of animation
-    connect(this, SIGNAL(updateUI()), SLOT(slotUpdateAnimation()));
-
     emit updateUI();
 }
 
@@ -88,10 +69,6 @@ KDebugger::~KDebugger()
 	m_programConfig->sync();
 	delete m_programConfig;
     }
-    // delete breakpoint objects
-    for (int i = m_brkpts.size()-1; i >= 0; i--) {
-	delete m_brkpts[i];
-    }
 
     delete m_typeTable;
 }
@@ -112,10 +89,12 @@ void KDebugger::restoreSettings(KConfig* /*config*/)
 const char GeneralGroup[] = "General";
 const char DebuggerCmdStr[] = "DebuggerCmdStr";
 const char TTYLevelEntry[] = "TTYLevel";
+const char KDebugger::DriverNameEntry[] = "DriverName";
 
-bool KDebugger::debugProgram(const QString& name)
+bool KDebugger::debugProgram(const QString& name,
+			     DebuggerDriver* driver)
 {
-    if (m_d->isRunning())
+    if (m_d != 0 && m_d->isRunning())
     {
 	QApplication::setOverrideCursor(waitCursor);
 
@@ -128,8 +107,21 @@ bool KDebugger::debugProgram(const QString& name)
 	    TRACE("timed out while waiting for gdb to die!");
 	    return false;
 	}
+	delete m_d;
+	m_d = 0;
     }
 
+    // wire up the driver
+    connect(driver, SIGNAL(activateFileLine(const QString&,int,const DbgAddr&)),
+	    this, SIGNAL(activateFileLine(const QString&,int,const DbgAddr&)));
+    connect(driver, SIGNAL(processExited(KProcess*)), SLOT(gdbExited(KProcess*)));
+    connect(driver, SIGNAL(commandReceived(CmdQueueItem*,const char*)),
+	    SLOT(parse(CmdQueueItem*,const char*)));
+    connect(driver, SIGNAL(wroteStdin(KProcess*)), SIGNAL(updateUI()));
+    connect(driver, SIGNAL(inferiorRunning()), SLOT(slotInferiorRunning()));
+    connect(driver, SIGNAL(enterIdleState()), SLOT(backgroundUpdate()));
+    connect(driver, SIGNAL(enterIdleState()), SIGNAL(updateUI()));
+
     // create the program settings object
     openProgramConfig(name);
 
@@ -142,8 +134,11 @@ bool KDebugger::debugProgram(const QString& name)
     }
     // the rest is read in later in the handler of DCexecutable
 
+    m_d = driver;
+
     if (!startDriver()) {
 	TRACE("startDriver failed");
+	m_d = 0;
 	return false;
     }
 
@@ -172,7 +167,7 @@ bool KDebugger::debugProgram(const QString& name)
 void KDebugger::shutdown()
 {
     // shut down debugger driver
-    if (m_d->isRunning())
+    if (m_d != 0 && m_d->isRunning())
     {
 	stopDriver();
     }
@@ -187,6 +182,11 @@ void KDebugger::useCoreFile(QString corefile, bool batch)
     }
 }
 
+void KDebugger::setAttachPid(const QString& pid)
+{
+    m_attachedPid = pid;
+}
+
 void KDebugger::programRun()
 {
     if (!isReady())
@@ -227,18 +227,34 @@ void KDebugger::programRunAgain()
     }
 }
 
-void KDebugger::programStep(bool byInsn)
+void KDebugger::programStep()
 {
     if (canSingleStep()) {
-	m_d->executeCmd(byInsn ? DCstepi : DCstep, true);
+	m_d->executeCmd(DCstep, true);
 	m_programRunning = true;
     }
 }
 
-void KDebugger::programNext(bool byInsn)
+void KDebugger::programNext()
 {
     if (canSingleStep()) {
-	m_d->executeCmd(byInsn ? DCnexti : DCnext, true);
+	m_d->executeCmd(DCnext, true);
+	m_programRunning = true;
+    }
+}
+
+void KDebugger::programStepi()
+{
+    if (canSingleStep()) {
+	m_d->executeCmd(DCstepi, true);
+	m_programRunning = true;
+    }
+}
+
+void KDebugger::programNexti()
+{
+    if (canSingleStep()) {
+	m_d->executeCmd(DCnexti, true);
 	m_programRunning = true;
     }
 }
@@ -268,9 +284,6 @@ bool KDebugger::runUntil(const QString& fileName, int lineNo)
     if (isReady() && m_programActive && !m_programRunning) {
 	// strip off directory part of file name
 	QString file = fileName;
-#if QT_VERSION < 200
-	file.detach();
-#endif
 	int offset = file.findRev("/");
 	if (offset >= 0) {
 	    file.remove(0, offset+1);
@@ -293,11 +306,14 @@ void KDebugger::programBreak()
 void KDebugger::programArgs(QWidget* parent)
 {
     if (m_haveExecutable) {
-	PgmArgs dlg(parent, m_executable, m_envVars);
+	QStringList allOptions = m_d->boolOptionList();
+	PgmArgs dlg(parent, m_executable, m_envVars, allOptions);
 	dlg.setArgs(m_programArgs);
 	dlg.setWd(m_programWD);
+	dlg.setOptions(m_boolOptions);
 	if (dlg.exec()) {
-	    updateProgEnvironment(dlg.args(), dlg.wd(), dlg.envVars());
+	    updateProgEnvironment(dlg.args(), dlg.wd(),
+				  dlg.envVars(), dlg.options());
 	}
     }
 }
@@ -334,24 +350,19 @@ bool KDebugger::setBreakpoint(QString file, int lineNo,
 	 * set the breakpoint exactly there. Otherwise we use the file name
 	 * plus line no.
 	 */
+	Breakpoint* bp = new Breakpoint;
+	bp->temporary = temporary;
+
 	if (address.isEmpty())
 	{
-	    // strip off directory part of file name
-#if QT_VERSION < 200
-	    file.detach();
-#endif
-	    int offset = file.findRev("/");
-	    if (offset >= 0) {
-		file.remove(0, offset+1);
-	    }
-	    m_d->executeCmd(temporary ? DCtbreakline : DCbreakline,
-			    file, lineNo);
+	    bp->fileName = file;
+	    bp->lineNo = lineNo;
 	}
 	else
 	{
-	    m_d->executeCmd(temporary ? DCtbreakaddr : DCbreakaddr,
-			    address.asString());
+	    bp->address = address;
 	}
+	setBreakpoint(bp, false);
     }
     else
     {
@@ -360,34 +371,148 @@ bool KDebugger::setBreakpoint(QString file, int lineNo,
 	 * delete that breakpoint.
 	 */
 	if (bp->enabled) {
-	    m_d->executeCmd(DCdelete, bp->id);
+	    deleteBreakpoint(bp);
 	} else {
-	    m_d->executeCmd(DCenable, bp->id);
+	    enableDisableBreakpoint(bp);
 	}
     }
     return true;
 }
 
+void KDebugger::setBreakpoint(Breakpoint* bp, bool queueOnly)
+{
+    CmdQueueItem* cmd;
+    if (!bp->text.isEmpty())
+    {
+	/*
+	 * The breakpoint was set using the text box in the breakpoint
+	 * list. This is the only way in which watchpoints are set.
+	 */
+	if (bp->type == Breakpoint::watchpoint) {
+	    cmd = m_d->executeCmd(DCwatchpoint, bp->text);
+	} else {
+	    cmd = m_d->executeCmd(DCbreaktext, bp->text);
+	}
+    }
+    else if (bp->address.isEmpty())
+    {
+	// strip off directory part of file name
+	QString file = bp->fileName;
+	int offset = file.findRev("/");
+	if (offset >= 0) {
+	    file.remove(0, offset+1);
+	}
+	if (queueOnly) {
+	    cmd = m_d->queueCmd(bp->temporary ? DCtbreakline : DCbreakline,
+				file, bp->lineNo, DebuggerDriver::QMoverride);
+	} else {
+	    cmd = m_d->executeCmd(bp->temporary ? DCtbreakline : DCbreakline,
+				  file, bp->lineNo);
+	}
+    }
+    else
+    {
+	if (queueOnly) {
+	    cmd = m_d->queueCmd(bp->temporary ? DCtbreakaddr : DCbreakaddr,
+				bp->address.asString(), DebuggerDriver::QMoverride);
+	} else {
+	    cmd = m_d->executeCmd(bp->temporary ? DCtbreakaddr : DCbreakaddr,
+				  bp->address.asString());
+	}
+    }
+    cmd->m_brkpt = bp;	// used in newBreakpoint()
+}
+
 bool KDebugger::enableDisableBreakpoint(QString file, int lineNo,
 					const DbgAddr& address)
 {
-    if (!isReady()) {
-	return false;
-    }
-
     Breakpoint* bp = breakpointByFilePos(file, lineNo, address);
-    if (bp == 0)
-	return true;
+    return bp == 0 || enableDisableBreakpoint(bp);
+}
 
-    // toggle enabled/disabled state
-    if (bp->enabled) {
-	m_d->executeCmd(DCdisable, bp->id);
+bool KDebugger::enableDisableBreakpoint(Breakpoint* bp)
+{
+    /*
+     * Toggle enabled/disabled state.
+     * 
+     * The driver is not bothered if we are modifying an orphaned
+     * breakpoint.
+     */
+    if (!bp->isOrphaned()) {
+	if (!canChangeBreakpoints()) {
+	    return false;
+	}
+	m_d->executeCmd(bp->enabled ? DCdisable : DCenable, bp->id);
     } else {
-	m_d->executeCmd(DCenable, bp->id);
+	bp->enabled = !bp->enabled;
+	emit breakpointsChanged();
     }
     return true;
 }
 
+bool KDebugger::conditionalBreakpoint(Breakpoint* bp,
+				      const QString& condition,
+				      int ignoreCount)
+{
+    /*
+     * Change the condition and ignore count.
+     *
+     * The driver is not bothered if we are removing an orphaned
+     * breakpoint.
+     */
+
+    if (!bp->isOrphaned()) {
+	if (!canChangeBreakpoints()) {
+	    return false;
+	}
+
+	bool changed = false;
+
+	if (bp->condition != condition) {
+	    // change condition
+	    m_d->executeCmd(DCcondition, condition, bp->id);
+	    changed = true;
+	}
+	if (bp->ignoreCount != ignoreCount) {
+	    // change ignore count
+	    m_d->executeCmd(DCignore, bp->id, ignoreCount);
+	    changed = true;
+	}
+	if (changed) {
+	    // get the changes
+	    m_d->queueCmd(DCinfobreak, DebuggerDriver::QMoverride);
+	}
+    } else {
+	bp->condition = condition;
+	bp->ignoreCount = ignoreCount;
+	emit breakpointsChanged();
+    }
+    return true;
+}
+
+bool KDebugger::deleteBreakpoint(Breakpoint* bp)
+{
+    /*
+     * Remove the breakpoint.
+     *
+     * The driver is not bothered if we are removing an orphaned
+     * breakpoint.
+     */
+    if (!bp->isOrphaned()) {
+	if (!canChangeBreakpoints()) {
+	    return false;
+	}
+	m_d->executeCmd(DCdelete, bp->id);
+    } else {
+	// move the last entry to bp's slot and shorten the list
+	int i = m_brkpts.findRef(bp);
+	m_brkpts.insert(i, m_brkpts.take(m_brkpts.size()-1));
+	m_brkpts.resize(m_brkpts.size()-1);
+	emit breakpointsChanged();
+    }
+    return false;
+}
+
 bool KDebugger::canSingleStep()
 {
     return isReady() && m_programActive && !m_programRunning;
@@ -401,12 +526,12 @@ bool KDebugger::canChangeBreakpoints()
 bool KDebugger::isReady() const 
 {
     return m_haveExecutable &&
-	m_d->canExecuteImmediately();
+	m_d != 0 && m_d->canExecuteImmediately();
 }
 
 bool KDebugger::isIdle() const
 {
-    return m_d->isIdle();
+    return m_d == 0 || m_d->isIdle();
 }
 
 
@@ -422,10 +547,8 @@ bool KDebugger::startDriver()
      * (which might also be empty, in which case the driver uses its
      * default).
      */
-    QString debuggerCmd = m_debuggerCmd.isEmpty()  ?
-	m_generalDebuggerCmd  :  m_debuggerCmd;
     m_explicitKill = false;
-    if (!m_d->startup(debuggerCmd)) {
+    if (!m_d->startup(m_debuggerCmd)) {
 	return false;
     }
 
@@ -502,15 +625,11 @@ void KDebugger::gdbExited(KProcess*)
     m_typeTable = 0;
 
     if (m_explicitKill) {
-	TRACE("gdb exited normally");
+	TRACE(m_d->driverName() + " exited normally");
     } else {
-	QString msg = i18n("gdb exited unexpectedly.\n"
+	QString msg = i18n("%1 exited unexpectedly.\n"
 			   "Restart the session (e.g. with File|Executable).");
-#if QT_VERSION < 200
-	KMsgBox::message(parentWidget(), kapp->appName(), msg, KMsgBox::EXCLAMATION);
-#else
-	KMessageBox::error(parentWidget(), msg);
-#endif
+	KMessageBox::error(parentWidget(), msg.arg(m_d->driverName()));
     }
 
     // reset state
@@ -522,16 +641,14 @@ void KDebugger::gdbExited(KProcess*)
     m_debuggerCmd = QString();		/* use global setting at next start! */
     m_attachedPid = QString();		/* we are no longer attached to a process */
     m_ttyLevel = ttyFull;
+    m_brkpts.clear();
 
-    // stop gear wheel and erase PC
-    stopAnimation();
+    // erase PC
     emit updatePC(QString(), -1, DbgAddr(), 0);
 }
 
-void KDebugger::openProgramConfig(const QString& name)
+QString KDebugger::getConfigForExe(const QString& name)
 {
-    ASSERT(m_programConfig == 0);
-
     QFileInfo fi(name);
     QString pgmConfigFile = fi.dirPath(true);
     if (!pgmConfigFile.isEmpty()) {
@@ -539,6 +656,14 @@ void KDebugger::openProgramConfig(const QString& name)
     }
     pgmConfigFile += ".kdbgrc." + fi.fileName();
     TRACE("program config file = " + pgmConfigFile);
+    return pgmConfigFile;
+}
+
+void KDebugger::openProgramConfig(const QString& name)
+{
+    ASSERT(m_programConfig == 0);
+
+    QString pgmConfigFile = getConfigForExe(name);
 
     m_programConfig = new ProgramConfig(pgmConfigFile);
 }
@@ -548,6 +673,7 @@ const char WatchGroup[] = "Watches";
 const char FileVersion[] = "FileVersion";
 const char ProgramArgs[] = "ProgramArgs";
 const char WorkingDirectory[] = "WorkingDirectory";
+const char OptionsSelected[] = "OptionsSelected";
 const char Variable[] = "Var%d";
 const char Value[] = "Value%d";
 const char ExprFmt[] = "Expr%d";
@@ -559,8 +685,13 @@ void KDebugger::saveProgramSettings()
     m_programConfig->writeEntry(FileVersion, 1);
     m_programConfig->writeEntry(ProgramArgs, m_programArgs);
     m_programConfig->writeEntry(WorkingDirectory, m_programWD);
+    m_programConfig->writeEntry(OptionsSelected, m_boolOptions);
     m_programConfig->writeEntry(DebuggerCmdStr, m_debuggerCmd);
     m_programConfig->writeEntry(TTYLevelEntry, int(m_ttyLevel));
+    QString driverName;
+    if (m_d != 0)
+	driverName = m_d->driverName();
+    m_programConfig->writeEntry(DriverNameEntry, driverName);
 
     // write environment variables
     m_programConfig->deleteGroup(EnvironmentGroup);
@@ -606,6 +737,8 @@ void KDebugger::restoreProgramSettings()
     // m_ttyLevel has been read in already
     QString pgmArgs = m_programConfig->readEntry(ProgramArgs);
     QString pgmWd = m_programConfig->readEntry(WorkingDirectory);
+    QStringList boolOptions = m_programConfig->readListEntry(OptionsSelected);
+    m_boolOptions = QStringList();
 
     // read environment variables
     m_programConfig->setGroup(EnvironmentGroup);
@@ -632,7 +765,7 @@ void KDebugger::restoreProgramSettings()
 	pgmVars.insert(name, var);
     }
 
-    updateProgEnvironment(pgmArgs, pgmWd, pgmVars);
+    updateProgEnvironment(pgmArgs, pgmWd, pgmVars, boolOptions);
 
     restoreBreakpoints(m_programConfig);
 
@@ -690,6 +823,7 @@ QString KDebugger::readDebuggerCmd()
 const char BPGroup[] = "Breakpoint %d";
 const char File[] = "File";
 const char Line[] = "Line";
+const char Text[] = "Text";
 const char Address[] = "Address";
 const char Temporary[] = "Temporary";
 const char Enabled[] = "Enabled";
@@ -704,26 +838,32 @@ void KDebugger::saveBreakpoints(ProgramConfig* config)
 	if (bp->type == Breakpoint::watchpoint)
 	    continue;			/* don't save watchpoints */
 	groupName.sprintf(BPGroup, i++);
+
+	/* remove remmants */
+	config->deleteGroup(groupName);
+
 	config->setGroup(groupName);
-	if (!bp->fileName.isEmpty()) {
+	if (!bp->text.isEmpty()) {
+	    /*
+	     * The breakpoint was set using the text box in the breakpoint
+	     * list. We do not save the location by filename+line number,
+	     * but instead honor what the user typed (a function name, for
+	     * example, which could move between sessions).
+	     */
+	    config->writeEntry(Text, bp->text);
+	} else if (!bp->fileName.isEmpty()) {
 	    config->writeEntry(File, bp->fileName);
 	    config->writeEntry(Line, bp->lineNo);
 	    /*
-	     * Addresses are hardly correct across sessions, so we remove
-	     * it since we have a file name and line number.
+	     * Addresses are hardly correct across sessions, so we don't
+	     * save it.
 	     */
-	    config->deleteEntry(Address, false);
 	} else {
 	    config->writeEntry(Address, bp->address.asString());
-	    /* remove remmants */
-	    config->deleteEntry(File, false);
-	    config->deleteEntry(Line, false);
 	}
 	config->writeEntry(Temporary, bp->temporary);
 	config->writeEntry(Enabled, bp->enabled);
-	if (bp->condition.isEmpty())
-	    config->deleteEntry(Condition, false);
-	else
+	if (!bp->condition.isEmpty())
 	    config->writeEntry(Condition, bp->condition);
 	// we do not save the ignore count
     }
@@ -743,11 +883,6 @@ void KDebugger::saveBreakpoints(ProgramConfig* config)
 void KDebugger::restoreBreakpoints(ProgramConfig* config)
 {
     QString groupName;
-    QString fileName;
-    QString address;
-    int lineNo;
-    bool enabled, temporary;
-    QString condition;
     /*
      * We recognize the end of the list if there is no Enabled entry
      * present.
@@ -759,35 +894,29 @@ void KDebugger::restoreBreakpoints(ProgramConfig* config)
 	    /* group not present, assume that we've hit them all */
 	    break;
 	}
-	fileName = config->readEntry(File);
-	lineNo = config->readNumEntry(Line, -1);
-	address = config->readEntry(Address);
-	if ((fileName.isEmpty() || lineNo < 0) && address.isEmpty())
+	Breakpoint* bp = new Breakpoint;
+	bp->fileName = config->readEntry(File);
+	bp->lineNo = config->readNumEntry(Line, -1);
+	bp->text = config->readEntry(Text);
+	bp->address = config->readEntry(Address);
+	// check consistency
+	if ((bp->fileName.isEmpty() || bp->lineNo < 0) &&
+	    bp->text.isEmpty() &&
+	    bp->address.isEmpty())
+	{
+	    delete bp;
 	    continue;
-	enabled = config->readBoolEntry(Enabled, true);
-	temporary = config->readBoolEntry(Temporary, false);
-	condition = config->readEntry(Condition);
+	}
+	bp->enabled = config->readBoolEntry(Enabled, true);
+	bp->temporary = config->readBoolEntry(Temporary, false);
+	bp->condition = config->readEntry(Condition);
+
 	/*
-	 * Add the breakpoint. We assume that we have started a new
-	 * instance of gdb, because we assign the breakpoint ids ourselves,
-	 * starting with 1. Then we use this id to disable the breakpoint,
-	 * if necessary. If this assignment of ids doesn't work, (maybe
-	 * because this isn't a fresh gdb at all), we disable the wrong
-	 * breakpoint! Oh well... for now it works.
+	 * Add the breakpoint.
 	 */
-	if (!fileName.isEmpty()) {
-	    m_d->executeCmd(temporary ? DCtbreakline : DCbreakline,
-			    fileName, lineNo);
-	} else {
-	    m_d->executeCmd(temporary ? DCtbreakaddr : DCbreakaddr,
-			    address);
-	}
-	if (!enabled) {
-	    m_d->executeCmd(DCdisable, i+1);
-	}
-	if (!condition.isEmpty()) {
-	    m_d->executeCmd(DCcondition, condition, i+1);
-	}
+	setBreakpoint(bp, false);
+	// the new breakpoint is disabled or conditionalized later
+	// in newBreakpoint()
     }
     m_d->queueCmd(DCinfobreak, DebuggerDriver::QMoverride);
 }
@@ -808,6 +937,7 @@ void KDebugger::parse(CmdQueueItem* cmd, const char* output)
 	// there is no output
     case DCsetenv:
     case DCunsetenv:
+    case DCsetoption:
 	/* if value is empty, we see output, but we don't care */
 	break;
     case DCcd:
@@ -825,23 +955,30 @@ void KDebugger::parse(CmdQueueItem* cmd, const char* output)
 		restoreProgramSettings();
 	    }
 	    // load file containing main() or core file
-	    if (m_corefile.isEmpty()) {
-		if (m_remoteDevice.isEmpty())
-		    m_d->queueCmd(DCinfolinemain, DebuggerDriver::QMnormal);
-	    } else {
+	    if (!m_corefile.isEmpty())
+	    {
 		// load core file
 		loadCoreFile();
 	    }
+	    else if (!m_attachedPid.isEmpty())
+	    {
+		m_d->queueCmd(DCattach, m_attachedPid, DebuggerDriver::QMoverride);
+		m_programActive = true;
+		m_programRunning = true;
+	    }
+	    else if (!m_remoteDevice.isEmpty())
+	    {
+		// handled elsewhere
+	    }
+	    else
+	    {
+		m_d->queueCmd(DCinfolinemain, DebuggerDriver::QMnormal);
+	    }
 	    if (!m_statusMessage.isEmpty())
 		emit updateStatusMessage();
 	} else {
-	    QString msg = "gdb: " + m_statusMessage;
-#if QT_VERSION < 200
-	    KMsgBox::message(parentWidget(), kapp->appName(), msg,
-			     KMsgBox::STOP, i18n("OK"));
-#else
+	    QString msg = m_d->driverName() + ": " + m_statusMessage;
 	    KMessageBox::sorry(parentWidget(), msg);
-#endif
 	    m_executable = "";
 	    m_corefile = "";		/* don't process core file */
 	    m_haveExecutable = false;
@@ -858,12 +995,8 @@ void KDebugger::parse(CmdQueueItem* cmd, const char* output)
 	} else {
 	    // report error
 	    QString msg = m_d->driverName() + ": " + QString(output);
-#if QT_VERSION < 200
-	    KMsgBox::message(parentWidget(), kapp->appName(), msg,
-			     KMsgBox::EXCLAMATION, i18n("OK"));
-#else
 	    KMessageBox::sorry(parentWidget(), msg);
-#endif
+
 	    // if core file was loaded from command line, revert to info line main
 	    if (!cmd->m_byUser) {
 		m_d->queueCmd(DCinfolinemain, DebuggerDriver::QMnormal);
@@ -904,7 +1037,12 @@ void KDebugger::parse(CmdQueueItem* cmd, const char* output)
     case DCprint:
 	handlePrint(cmd, output);
 	break;
+    case DCprintDeref:
+	handlePrintDeref(cmd, output);
+	break;
     case DCattach:
+	m_haveExecutable = true;
+	// fall through
     case DCrun:
     case DCcont:
     case DCstep:
@@ -927,7 +1065,7 @@ void KDebugger::parse(CmdQueueItem* cmd, const char* output)
     case DCbreakaddr:
     case DCtbreakaddr:
     case DCwatchpoint:
-	newBreakpoint(output);
+	newBreakpoint(cmd, output);
 	// fall through
     case DCdelete:
     case DCenable:
@@ -939,7 +1077,6 @@ void KDebugger::parse(CmdQueueItem* cmd, const char* output)
 	// note: this handler must not enqueue a command, since
 	// DCinfobreak is used at various different places.
 	updateBreakList(output);
-	emit lineItemsChanged();
 	break;
     case DCfindType:
 	handleFindType(cmd, output);
@@ -958,6 +1095,12 @@ void KDebugger::parse(CmdQueueItem* cmd, const char* output)
     case DCinfothreads:
 	handleThreadList(output);
 	break;
+    case DCsetpc:
+	handleSetPC(output);
+	break;
+    case DCsetvariable:
+	handleSetVariable(cmd, output);
+	break;
     }
 }
 
@@ -983,6 +1126,19 @@ void KDebugger::handleRunCommands(const char* output)
 	emit executableUpdated();
     }
 
+    /* 
+     * Try to set any orphaned breakpoints now.
+     */
+    for (int i = m_brkpts.size()-1; i >= 0; i--) {
+	if (m_brkpts[i]->isOrphaned()) {
+	    TRACE("re-trying brkpt loc: "+m_brkpts[i]->location+
+		  " file: "+m_brkpts[i]->fileName+
+		  QString().sprintf(" line: %d", m_brkpts[i]->lineNo));
+	    setBreakpoint(m_brkpts[i], true);
+	    flags |= DebuggerDriver::SFrefreshBreak;
+	}
+    }
+
     /*
      * If we stopped at a breakpoint, we must update the breakpoint list
      * because the hit count changes. Also, if the breakpoint was temporary
@@ -1015,7 +1171,7 @@ void KDebugger::handleRunCommands(const char* output)
     }
 
     /* Update threads list */
-    if (flags & DebuggerDriver::SFrefreshThreads) {
+    if (m_programActive && (flags & DebuggerDriver::SFrefreshThreads)) {
 	m_d->queueCmd(DCinfothreads, DebuggerDriver::QMoverride);
     }
 
@@ -1052,7 +1208,8 @@ void KDebugger::updateAllExprs()
 }
 
 void KDebugger::updateProgEnvironment(const QString& args, const QString& wd,
-				      const QDict<EnvVar>& newVars)
+				      const QDict<EnvVar>& newVars,
+				      const QStringList& newOptions)
 {
     m_programArgs = args;
     m_d->executeCmd(DCsetargs, m_programArgs);
@@ -1064,6 +1221,7 @@ void KDebugger::updateProgEnvironment(const QString& args, const QString& wd,
 	TRACE("new wd: " + m_programWD + "\n");
     }
 
+    // update environment variables
     QDictIterator<EnvVar> it = newVars;
     EnvVar* val;
     for (; (val = it) != 0; ++it) {
@@ -1092,6 +1250,29 @@ void KDebugger::updateProgEnvironment(const QString& args, const QString& wd,
 	    break;
 	}
     }
+
+    // update options
+    QStringList::ConstIterator oi;
+    for (oi = newOptions.begin(); oi != newOptions.end(); ++oi)
+    {
+	if (m_boolOptions.findIndex(*oi) < 0) {
+	    // the options is currently not set, so set it
+	    m_d->executeCmd(DCsetoption, *oi, 1);
+	} else {
+	    // option is set, no action required, but move it to the end
+	    m_boolOptions.remove(*oi);
+	}
+	m_boolOptions.append(*oi);
+    }
+    /*
+     * Now all options that should be set are at the end of m_boolOptions.
+     * If some options need to be unset, they are at the front of the list.
+     * Here we unset and remove them.
+     */
+    while (m_boolOptions.count() > newOptions.count()) {
+	m_d->executeCmd(DCsetoption, m_boolOptions.first(), 0);
+	m_boolOptions.remove(m_boolOptions.begin());
+    }
 }
 
 void KDebugger::handleLocals(const char* output)
@@ -1139,9 +1320,7 @@ void KDebugger::handleLocals(const char* output)
 	    // remove the new variable from the list
 	    newVars.remove();
 	    delete v;
-#if QT_VERSION >= 200
 	    repaintNeeded = true;
-#endif
 	}
     }
     // insert all remaining new variables
@@ -1202,7 +1381,29 @@ bool KDebugger::handlePrint(CmdQueueItem* cmd, const char* output)
     // set expression "name"
     variable->setText(cmd->m_expr->getText());
 
-    if (cmd->m_expr->m_varKind == VarTree::VKpointer) {
+    {
+	TRACE("update expr: " + cmd->m_expr->getText());
+	cmd->m_exprWnd->updateExpr(cmd->m_expr, variable);
+	delete variable;
+    }
+
+    evalExpressions();			/* enqueue dereferenced pointers */
+
+    return true;
+}
+
+bool KDebugger::handlePrintDeref(CmdQueueItem* cmd, const char* output)
+{
+    ASSERT(cmd->m_expr != 0);
+
+    VarTree* variable = parseExpr(output, true);
+    if (variable == 0)
+	return false;
+
+    // set expression "name"
+    variable->setText(cmd->m_expr->getText());
+
+    {
 	/*
 	 * We must insert a dummy parent, because otherwise variable's value
 	 * would overwrite cmd->m_expr's value.
@@ -1221,10 +1422,6 @@ bool KDebugger::handlePrint(CmdQueueItem* cmd, const char* output)
 	TRACE("update ptr: " + cmd->m_expr->getText());
 	cmd->m_exprWnd->updateExpr(cmd->m_expr, dummyParent);
 	delete dummyParent;
-    } else {
-	TRACE("update expr: " + cmd->m_expr->getText());
-	cmd->m_exprWnd->updateExpr(cmd->m_expr, variable);
-	delete variable;
     }
 
     evalExpressions();			/* enqueue dereferenced pointers */
@@ -1379,12 +1576,11 @@ void KDebugger::dereferencePointer(ExprWnd* wnd, VarTree* exprItem,
 
     QString expr = exprItem->computeExpr();
     TRACE("dereferencing pointer: " + expr);
-    QString queueExpr = "*(" + expr + ")";
     CmdQueueItem* cmd;
     if (immediate) {
-	cmd = m_d->queueCmd(DCprint, queueExpr, DebuggerDriver::QMoverrideMoreEqual);
+	cmd = m_d->queueCmd(DCprintDeref, expr, DebuggerDriver::QMoverrideMoreEqual);
     } else {
-	cmd = m_d->queueCmd(DCprint, queueExpr, DebuggerDriver::QMoverride);
+	cmd = m_d->queueCmd(DCprintDeref, expr, DebuggerDriver::QMoverride);
     }
     // remember which expr this was
     cmd->m_expr = exprItem;
@@ -1491,9 +1687,6 @@ void KDebugger::handlePrintStruct(CmdQueueItem* cmd, const char* output)
     if (errorValue || !var->m_exprIndexUseGuard)
     {
 	// add current partValue (which might be the question marks)
-#if QT_VERSION < 200
-	var->m_partialValue.detach();
-#endif
 	var->m_partialValue += partValue;
 	var->m_exprIndex++;		/* next part */
 	var->m_exprIndexUseGuard = true;
@@ -1600,6 +1793,9 @@ void KDebugger::handleSharedLibs(const char* output)
 
     // get type libraries
     m_typeTable->loadLibTypes(m_sharedLibs);
+
+    // hand over the QString data cmd
+    m_d->setPrintQStringDataCmd(m_typeTable->printQStringDataCmd());
 }
 
 CmdQueueItem* KDebugger::loadCoreFile()
@@ -1668,39 +1864,6 @@ void KDebugger::slotDeleteWatch()
     // item is invalid at this point!
 }
 
-void KDebugger::startAnimation(bool fast)
-{
-    int interval = fast ? 50 : 150;
-    if (!m_animationTimer.isActive()) {
-	m_animationTimer.start(interval);
-    } else if (m_animationInterval != interval) {
-	m_animationTimer.changeInterval(interval);
-    }
-    m_animationInterval = interval;
-}
-
-void KDebugger::stopAnimation()
-{
-    if (m_animationTimer.isActive()) {
-	m_animationTimer.stop();
-	m_animationInterval = 0;
-    }
-}
-
-void KDebugger::slotUpdateAnimation()
-{
-    if (m_d->isIdle()) {
-	stopAnimation();
-    } else {
-	/*
-	 * Slow animation while program is stopped (i.e. while variables
-	 * are displayed)
-	 */
-	bool slow = isReady() && m_programActive && !m_programRunning;
-	startAnimation(!slow);
-    }
-}
-
 void KDebugger::handleRegisters(const char* output)
 {
     QList<RegisterInfo> regs;
@@ -1715,36 +1878,73 @@ void KDebugger::handleRegisters(const char* output)
 /*
  * The output of the DCbreak* commands has more accurate information about
  * the file and the line number.
+ *
+ * All newly set breakpoints are inserted in the m_brkpts, even those that
+ * were not set sucessfully. The unsuccessful breakpoints ("orphaned
+ * breakpoints") are assigned negative ids, and they are tried to set later
+ * when the program stops again at a breakpoint.
  */
-void KDebugger::newBreakpoint(const char* output)
+void KDebugger::newBreakpoint(CmdQueueItem* cmd, const char* output)
 {
+    Breakpoint* bp = cmd->m_brkpt;
+    assert(bp != 0);
+    if (bp == 0)
+	return;
+
+    // if this is a new breakpoint, put it in the list
+    bool isNew = !m_brkpts.contains(bp);
+    if (isNew) {
+	assert(bp->id == 0);
+	int n = m_brkpts.size();
+	m_brkpts.resize(n+1);
+	m_brkpts.insert(n, bp);
+    }
+
+    // parse the output to determine success or failure
     int id;
     QString file;
     int lineNo;
-    if (!m_d->parseBreakpoint(output, id, file, lineNo))
+    QString address;
+    if (!m_d->parseBreakpoint(output, id, file, lineNo, address))
+    {
+	/*
+	 * Failure, the breakpoint could not be set. If this is a new
+	 * breakpoint, assign it a negative id. We look for the minimal id
+	 * of all breakpoints (that are already in the list) to get the new
+	 * id.
+	 */
+	if (isNew)
+	{
+	    assert(bp->id == 0);
+	    for (int i = m_brkpts.size()-2; i >= 0; i--) {
+		if (m_brkpts[i]->id < bp->id) {
+		    bp->id = m_brkpts[i]->id;
+		    break;
+		}
+	    }
+	    --bp->id;
+	}
 	return;
+    }
 
-    // see if it is new
-    for (int i = m_brkpts.size()-1; i >= 0; i--) {
-	if (m_brkpts[i]->id == id) {
-	    // not new; update
-	    m_brkpts[i]->fileName = file;
-	    m_brkpts[i]->lineNo = lineNo;
-	    return;
+    // The breakpoint was successfully set.
+    if (bp->id <= 0)
+    {
+	// this is a new or orphaned breakpoint:
+	// set the remaining properties
+	if (!cmd->m_brkpt->enabled) {
+	    m_d->executeCmd(DCdisable, id);
+	}
+	if (!cmd->m_brkpt->condition.isEmpty()) {
+	    m_d->executeCmd(DCcondition, cmd->m_brkpt->condition, id);
 	}
     }
-    // yes, new
-    Breakpoint* bp = new Breakpoint;
+
     bp->id = id;
-    bp->temporary = false;
-    bp->enabled = true;
-    bp->hitCount = 0;
-    bp->ignoreCount = 0;
     bp->fileName = file;
     bp->lineNo = lineNo;
-    int n = m_brkpts.size();
-    m_brkpts.resize(n+1);
-    m_brkpts[n] = bp;
+    if (!address.isEmpty())
+	bp->address = address;
 }
 
 void KDebugger::updateBreakList(const char* output)
@@ -1756,35 +1956,56 @@ void KDebugger::updateBreakList(const char* output)
 
     // merge new information into existing breakpoints
 
-    QArray<Breakpoint*> oldbrks = m_brkpts;
-
-    // move parsed breakpoints into m_brkpts
-    m_brkpts.detach();
-    m_brkpts.resize(brks.count());
-    int n = 0;
-    for (Breakpoint* bp = brks.first(); bp != 0; bp = brks.next())
+    for (int i = m_brkpts.size()-1; i >= 0; i--)	// decrement!
     {
-	m_brkpts[n++] = bp;
+	// skip orphaned breakpoints
+	if (m_brkpts[i]->id < 0)
+	    continue;
+
+	for (Breakpoint* bp = brks.first(); bp != 0; bp = brks.next())
+	{
+	    if (bp->id == m_brkpts[i]->id) {
+		// keep accurate location
+		// except that xsldbg doesn't have a location in
+		// the old breakpoint if it's just been set
+		bp->text = m_brkpts[i]->text;
+		if (!m_brkpts[i]->fileName.isEmpty()) {
+		    bp->fileName = m_brkpts[i]->fileName;
+		    bp->lineNo = m_brkpts[i]->lineNo;
+		}
+		m_brkpts.insert(i, bp); // old object is deleted
+		goto stillAlive;
+	    }
+	}
+	/*
+	 * If we get here, this breakpoint is no longer present.
+	 * 
+	 * To delete the breakpoint at i, we place the last breakpoint in
+	 * the list into the slot i. This will delete the old object at i.
+	 * Then we shorten the list by one.
+	 */
+	m_brkpts.insert(i, m_brkpts.take(m_brkpts.size()-1));
+	m_brkpts.resize(m_brkpts.size()-1);
+	TRACE(QString().sprintf("deleted brkpt %d, have now %d brkpts", i, m_brkpts.size()));
+
+	stillAlive:;
     }
 
-    // go through all old breakpoints
-    for (int i = oldbrks.size()-1; i >= 0; i--) {
-	// is this one still alive?
-	for (int j = m_brkpts.size()-1; j >= 0; j--)
-	{
-	    if (m_brkpts[j]->id == oldbrks[i]->id) {
-		// yes, it is
-		// keep accurate location
-		m_brkpts[j]->fileName = oldbrks[i]->fileName;
-		m_brkpts[j]->lineNo = oldbrks[i]->lineNo;
+    // brks may contain new breakpoints not already in m_brkpts
+    for (const Breakpoint* bp = brks.first(); bp != 0; bp = brks.next())
+    {
+	bool found = false;
+	for (uint i = 0; i < m_brkpts.size(); i++)      {
+	    if (bp->id == m_brkpts[i]->id) {
+		found = true;
 		break;
 	    }
 	}
-    }
-
-    // delete old breakpoints
-    for (int i = oldbrks.size()-1; i >= 0; i--) {
-	delete oldbrks[i];
+	if (!found){
+	    int n = m_brkpts.size();
+	    m_brkpts.resize(n+1);
+	    m_brkpts.insert(n, bp);
+	}
     }
 
     emit breakpointsChanged();
@@ -1818,9 +2039,6 @@ Breakpoint* KDebugger::breakpointByFilePos(QString file, int lineNo,
     // not found, so try basename
     // strip off directory part of file name
     int offset = file.findRev("/");
-#if QT_VERSION < 200
-    file.detach();
-#endif
     file.remove(0, offset+1);
 
     for (i = m_brkpts.size()-1; i >= 0; i--) {
@@ -1828,9 +2046,6 @@ Breakpoint* KDebugger::breakpointByFilePos(QString file, int lineNo,
 	QString basename = m_brkpts[i]->fileName;
 	int offset = basename.findRev("/");
 	if (offset >= 0) {
-#if QT_VERSION < 200
-	    basename.detach();
-#endif
 	    basename.remove(0, offset+1);
 	}
 
@@ -1846,6 +2061,18 @@ Breakpoint* KDebugger::breakpointByFilePos(QString file, int lineNo,
     return 0;
 }
 
+Breakpoint* KDebugger::breakpointById(int id)
+{
+    for (int i = m_brkpts.size()-1; i >= 0; i--)
+    {
+	if (m_brkpts[i]->id == id) {
+	    return m_brkpts[i];
+	}
+    }
+    // not found
+    return 0;
+}
+
 void KDebugger::slotValuePopup(const QString& expr)
 {
     // search the local variables for a match
@@ -1894,16 +2121,25 @@ void KDebugger::slotDisassemble(const QString& fileName, int lineNo)
 void KDebugger::handleInfoLine(CmdQueueItem* cmd, const char* output)
 {
     QString addrFrom, addrTo;
-    if (m_d->parseInfoLine(output, addrFrom, addrTo)) {
-	// got the address range, now get the real code
-	CmdQueueItem* c = m_d->queueCmd(DCdisassemble, addrFrom, addrTo,
-					DebuggerDriver::QMoverrideMoreEqual);
-	c->m_fileName = cmd->m_fileName;
-	c->m_lineNo = cmd->m_lineNo;
+    if (cmd->m_lineNo >= 0) {
+	// disassemble
+	if (m_d->parseInfoLine(output, addrFrom, addrTo)) {
+	    // got the address range, now get the real code
+	    CmdQueueItem* c = m_d->queueCmd(DCdisassemble, addrFrom, addrTo,
+					    DebuggerDriver::QMoverrideMoreEqual);
+	    c->m_fileName = cmd->m_fileName;
+	    c->m_lineNo = cmd->m_lineNo;
+	} else {
+	    // no code
+	    QList<DisassembledCode> empty;
+	    emit disassembled(cmd->m_fileName, cmd->m_lineNo, empty);
+	}
     } else {
-	// no code
-	QList<DisassembledCode> empty;
-	emit disassembled(cmd->m_fileName, cmd->m_lineNo, empty);
+	// set program counter
+	if (m_d->parseInfoLine(output, addrFrom, addrTo)) {
+	    // move the program counter to the start address
+	    m_d->executeCmd(DCsetpc, addrFrom);
+	}
     }
 }
 
@@ -1956,5 +2192,63 @@ void KDebugger::handleMemoryDump(const char* output)
     emit memoryDumpChanged(msg, memdump);
 }
 
+void KDebugger::setProgramCounter(const QString& file, int line, const DbgAddr& addr)
+{
+    if (addr.isEmpty()) {
+	// find address of the specified line
+	CmdQueueItem* cmd = m_d->executeCmd(DCinfoline, file, line);
+	cmd->m_lineNo = -1;		/* indicates "Set PC" UI command */
+    } else {
+	// move the program counter to that address
+	m_d->executeCmd(DCsetpc, addr.asString());
+    }
+}
+
+void KDebugger::handleSetPC(const char* /*output*/)
+{
+    // TODO: handle errors
+
+    // now go to the top-most frame
+    // this also modifies the program counter indicator in the UI
+    gotoFrame(0);
+}
+
+void KDebugger::slotValueEdited(int row, const QString& text)
+{
+    if (text.simplifyWhiteSpace().isEmpty())
+	return;			       /* no text entered: ignore request */
+
+    ASSERT(sender()->inherits("ExprWnd"));
+    ExprWnd* wnd = const_cast<ExprWnd*>(static_cast<const ExprWnd*>(sender()));
+    TRACE(QString().sprintf("Changing %s at row %d to ",
+			    wnd->name(), row) + text);
+
+    // determine the lvalue to edit
+    VarTree* expr = static_cast<VarTree*>(wnd->itemAt(row));
+    QString lvalue = expr->computeExpr();
+    CmdQueueItem* cmd = m_d->executeCmd(DCsetvariable, lvalue, text);
+    cmd->m_expr = expr;
+    cmd->m_exprWnd = wnd;
+}
+
+void KDebugger::handleSetVariable(CmdQueueItem* cmd, const char* output)
+{
+    QString msg = m_d->parseSetVariable(output);
+    if (!msg.isEmpty())
+    {
+	// there was an error; display it in the status bar
+	m_statusMessage = msg;
+	emit updateStatusMessage();
+	return;
+    }
+
+    // get the new value
+    QString expr = cmd->m_expr->computeExpr();
+    CmdQueueItem* printCmd =
+	m_d->queueCmd(DCprint, expr, DebuggerDriver::QMoverrideMoreEqual);
+    printCmd->m_expr = cmd->m_expr;
+    printCmd->m_exprWnd = cmd->m_exprWnd;
+}
+
 
 #include "debugger.moc"
