@@ -1,4 +1,4 @@
-// $Id: debugger.h,v 1.66.2.2 2003/08/16 16:22:29 jsixt Exp $
+// $Id: debugger.h,v 1.86 2005/01/01 21:23:31 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -8,6 +8,8 @@
 
 #include <qtimer.h>
 #include <qdict.h>
+#include <qptrvector.h>
+#include <qstringlist.h>
 #include "envvar.h"
 #include "exprwnd.h"			/* some compilers require this */
 
@@ -41,18 +43,19 @@ public:
     KDebugger(QWidget* parent,		/* will be used as the parent for dialogs */
 	      ExprWnd* localVars,
 	      ExprWnd* watchVars,
-	      QListBox* backtrace,
-	      DebuggerDriver* driver
-	      );
+	      QListBox* backtrace);
     ~KDebugger();
-    
+
     /**
-     * This function starts to debug the specified executable. If a program
-     * is currently being debugged, it is terminated first.
+     * This function starts to debug the specified executable using the
+     * specified driver. If a program is currently being debugged, it is
+     * terminated first. Ownership of driver is taken if and only if
+     * true is returned.
      *
      * @return false if an error occurs.
      */
-    bool debugProgram(const QString& executable);
+    bool debugProgram(const QString& executable,
+		      DebuggerDriver* driver);
 
     /**
      * Uses the specified core to debug the active program.
@@ -62,15 +65,32 @@ public:
     void useCoreFile(QString corefile, bool batch);
 
     /**
-     * Runs the program or continues it if it is stopped at a breakpoint.
+     * Uses the specified pid to attach to the active program.
      */
-    void programRun();
+    void setAttachPid(const QString& pid);
 
     /**
      * Attaches to the specified process and debugs it.
      */
     void attachProgram(const QString& pid);
 
+    /**
+     * Returns the file name of the per-program config file for the
+     * specified program.
+     */
+    static QString getConfigForExe(const QString& exe);
+
+    /**
+     * The driver name entry in the per-program config file.
+     */
+    static const char DriverNameEntry[];
+
+public slots:
+    /**
+     * Runs the program or continues it if it is stopped at a breakpoint.
+     */
+    void programRun();
+
     /**
      * Restarts the debuggee.
      */
@@ -80,13 +100,25 @@ public:
      * Performs a single-step, possibly stepping into a function call.
      * If byInsn is true, a step by instruction is performed.
      */
-    void programStep(bool byInsn);
+    void programStep();
 
     /**
      * Performs a single-step, stepping over a function call.
      * If byInsn is true, a step by instruction is performed.
      */
-    void programNext(bool byInsn);
+    void programNext();
+
+    /**
+     * Performs a single-step by instruction, possibly stepping into a
+     * function call.
+     */
+    void programStepi();
+
+    /**
+     * Performs a single-step by instruction, stepping over a function
+     * call.
+     */
+    void programNexti();
 
     /**
      * Runs the program until it returns from the current function.
@@ -103,6 +135,13 @@ public:
      */
     void programBreak();
 
+    /**
+     * Moves the program counter to the specified line.
+     * If an address is given, it is moved to the address.
+     */
+    void setProgramCounter(const QString&, int, const DbgAddr&);
+
+public:
     /**
      * Queries the user for program arguments.
      */
@@ -141,6 +180,14 @@ public:
     bool setBreakpoint(QString fileName, int lineNo,
 		       const DbgAddr& address, bool temporary);
 
+    /**
+     * Set a breakpoint.
+     * 
+     * @param bp Describes the breakpoint.
+     * @param queueOnly If false, the breakpoint is set using a high-priority command.
+     */
+    void setBreakpoint(Breakpoint* bp, bool queueOnly);
+
     /**
      * Enable or disable a breakpoint at the specified location.
      * 
@@ -153,6 +200,34 @@ public:
     bool enableDisableBreakpoint(QString fileName, int lineNo,
 				 const DbgAddr& address);
 
+    /**
+     * Enables or disables the specified breakpoint.
+     *
+     * @return false if the command was not executed, e.g. because the
+     * debuggee is running at the moment.
+     */
+    bool enableDisableBreakpoint(Breakpoint* bp);
+
+    /**
+     * Removes the specified breakpoint. Note that if bp is an orphaned
+     * breakpoint, then bp is an invalid pointer if (and only if) this
+     * function returns true.
+     *
+     * @return false if the command was not executed, e.g. because the
+     * debuggee is running at the moment.
+     */
+    bool deleteBreakpoint(Breakpoint* bp);
+
+    /**
+     * Changes the specified breakpoint's condition and ignore count.
+     *
+     * @return false if the command was not executed, e.g. because the
+     * debuggee is running at the moment.
+     */
+    bool conditionalBreakpoint(Breakpoint* bp,
+			       const QString& condition,
+			       int ignoreCount);
+
     /**
      * Tells whether one of the single stepping commands can be invoked
      * (step, next, finish, until, also run).
@@ -208,14 +283,12 @@ public:
     int numBreakpoints() const { return m_brkpts.size(); }
     const Breakpoint* breakpoint(int i) const { return m_brkpts[i]; }
 
-    const QString& executable() const { return m_executable; }
-
     /**
-     * Sets the command to invoke gdb. If cmd is the empty string, the
-     * default is substituted.
+     * Returns the breakpoint with the specified \a id.
      */
-    void setDebuggerCmd(const QString& cmd)
-    { m_generalDebuggerCmd = cmd; }
+    Breakpoint* breakpointById(int id);
+
+    const QString& executable() const { return m_executable; }
 
     /**
      * Terminal emulation level.
@@ -259,14 +332,13 @@ public:
 protected:
     QString m_inferiorTerminal;
     QString m_debuggerCmd;		/* per-program setting */
-    QString m_generalDebuggerCmd;	/* global setting */
     TTYLevel m_ttyLevel;		/* level of terminal emulation */
     bool startDriver();
     void stopDriver();
     void writeCommand();
     
     QList<VarTree> m_watchEvalExpr;	/* exprs to evaluate for watch windows */
-    QArray<Breakpoint*> m_brkpts;
+    QPtrVector<Breakpoint> m_brkpts;
     QString m_memoryExpression;		/* memory location to watch */
     unsigned m_memoryFormat;		/* how that output should look */
 
@@ -277,10 +349,12 @@ protected:
     void handleRunCommands(const char* output);
     void updateAllExprs();
     void updateProgEnvironment(const QString& args, const QString& wd,
-			       const QDict<EnvVar>& newVars);
+			       const QDict<EnvVar>& newVars,
+			       const QStringList& newOptions);
     void parseLocals(const char* output, QList<VarTree>& newVars);
     void handleLocals(const char* output);
     bool handlePrint(CmdQueueItem* cmd, const char* output);
+    bool handlePrintDeref(CmdQueueItem* cmd, const char* output);
     void handleBacktrace(const char* output);
     void handleFrameChange(const char* output);
     void handleFindType(CmdQueueItem* cmd, const char* output);
@@ -291,6 +365,8 @@ protected:
     void handleInfoLine(CmdQueueItem* cmd, const char* output);
     void handleDisassemble(CmdQueueItem* cmd, const char* output);
     void handleThreadList(const char* output);
+    void handleSetPC(const char* output);
+    void handleSetVariable(CmdQueueItem* cmd, const char* output);
     void evalExpressions();
     void evalInitialStructExpression(VarTree* var, ExprWnd* wnd, bool immediate);
     void evalStructExpression(VarTree* var, ExprWnd* wnd, bool immediate);
@@ -304,7 +380,7 @@ protected:
 
     Breakpoint* breakpointByFilePos(QString file, int lineNo,
 				    const DbgAddr& address);
-    void newBreakpoint(const char* output);
+    void newBreakpoint(CmdQueueItem* cmd, const char* output);
     void updateBreakList(const char* output);
     bool stopMayChangeBreakList() const;
     void saveBreakpoints(ProgramConfig* config);
@@ -321,6 +397,7 @@ protected:
     QString m_remoteDevice;
     QString m_programWD;		/* working directory of gdb */
     QDict<EnvVar> m_envVars;		/* environment variables set by user */
+    QStringList m_boolOptions;		/* boolean options */
     QStrList m_sharedLibs;		/* shared libraries used by program */
     ProgramTypeTable* m_typeTable;	/* known types used by the program */
     ProgramConfig* m_programConfig;	/* program-specific settings (brkpts etc) */
@@ -341,10 +418,10 @@ protected slots:
     void gotoFrame(int);
     void slotLocalsExpanding(KTreeViewItem*, bool&);
     void slotWatchExpanding(KTreeViewItem*, bool&);
-    void slotUpdateAnimation();
     void slotDeleteWatch();
     void slotValuePopup(const QString&);
     void slotDisassemble(const QString&, int);
+    void slotValueEdited(int, const QString&);
 public slots:
     void setThread(int);
     void shutdown();
@@ -372,14 +449,7 @@ signals:
     void activateFileLine(const QString& file, int lineNo, const DbgAddr& address);
 
     /**
-     * This signal is emitted when a line decoration item (those thingies
-     * that indicate breakpoints) must be changed.
-     */
-    void lineItemsChanged();
-
-    /**
-     * This signal is a special case of @ref #lineItemsChanged because it
-     * indicates that only the program counter has changed.
+     * This signal indicates that the program counter has changed.
      *
      * @param filename specifies the filename where the program stopped
      * @param lineNo specifies the line number (zero-based); it can be -1
@@ -401,12 +471,6 @@ signals:
      */
     void executableUpdated();
 
-    /**
-     * This signal is emitted when the animated icon should advance to the
-     * next picture.
-     */
-    void animationTimeout();
-
     /**
      * Indicates that a new status message is available.
      */
@@ -473,15 +537,8 @@ protected:
     ExprWnd& m_watchVariables;
     QListBox& m_btWindow;
 
-    // animation
-    QTimer m_animationTimer;
-    int m_animationInterval;
-    
     // implementation helpers
 protected:
-    void startAnimation(bool fast);
-    void stopAnimation();
-
     QWidget* parentWidget() { return static_cast<QWidget*>(parent()); }
 
     friend class BreakpointTable;
