@@ -1,4 +1,4 @@
-// $Id: exprwnd.cpp,v 1.19.2.5 2004/05/19 20:35:51 jsixt Exp $
+// $Id: exprwnd.cpp,v 1.26 2004/05/20 15:08:42 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -8,6 +8,7 @@
 #include "typetable.h"
 #include <qstrlist.h>
 #include <qpainter.h>
+#include <qscrollbar.h>
 #include <kapp.h>
 #include <kiconloader.h>		/* icons */
 #ifdef HAVE_CONFIG_H
@@ -242,19 +243,15 @@ TypeInfo* VarTree::inferTypeFromBaseClass()
 
 ExprWnd::ExprWnd(QWidget* parent, const char* name) :
 	KTreeView(parent, name),
-	maxValueWidth(0)
+	maxValueWidth(0),
+	m_edit(this)
 {
     setNumCols(2);
     
     connect(this, SIGNAL(expanded(int)), SLOT(slotExpandOrCollapse(int)));
     connect(this, SIGNAL(collapsed(int)), SLOT(slotExpandOrCollapse(int)));
 
-#if QT_VERSION < 200
-    KIconLoader* loader = kapp->getIconLoader();
-    m_pixPointer = loader->loadIcon("pointer.xpm");
-#else
     m_pixPointer = UserIcon("pointer.xpm");
-#endif
     if (m_pixPointer.isNull())
 	TRACE("Can't load pointer.xpm");
 }
@@ -680,3 +677,124 @@ void ExprWnd::slotExpandOrCollapse(int)
 {
     updateValuesWidth();
 }
+
+void ExprWnd::editValue(int row, const QString& text)
+{
+    int x;
+    colXPos(1, &x);
+    int y;
+    rowYPos(row, &y);
+    int w = cellWidth(1);
+    int h = cellHeight(row);
+    QScrollBar* sbV = static_cast<QScrollBar*>(child("table_sbV"));
+
+    /*
+     * Make the edit widget at least 5 characters wide (but not wider than
+     * this widget). If less than half of this widget is used to display
+     * the text, scroll this widget so that half of it shows the text (or
+     * less than half of it if the text is shorter).
+     */
+    QFontMetrics metr = m_edit.font();
+    int wMin = metr.width("88888");
+    if (w < wMin)
+	w = wMin;
+    int wThis = width();
+    if (sbV->isVisible())	// subtract width of scrollbar
+	wThis -= sbV->width();
+    if (x >= wThis/2 &&		// less than half the width displays text
+	x+w > wThis)		// not all text is visible
+    {
+	// scroll so that more text is visible
+	int wScroll = QMIN(x-wThis/2, x+w-wThis);
+	sbHor(xOffset()+wScroll);
+	colXPos(1, &x);
+    }
+    else if (x < 0)
+    {
+	// don't let the edit move out at the left
+	x = 0;
+    }
+
+    // make the edit box as wide as the visible column
+    QRect rect(x,y, wThis-x,h);
+    m_edit.setText(text);
+    m_edit.selectAll();
+
+    m_edit.setGeometry(rect);
+    m_edit.m_finished = false;
+    m_edit.m_row = row;
+    m_edit.show();
+    m_edit.setFocus();
+}
+
+bool ExprWnd::isEditing() const
+{
+    return m_edit.isVisible();
+}
+
+
+ValueEdit::ValueEdit(ExprWnd* parent) :
+	QLineEdit(parent, "valueedit")
+{
+    setFrame(false);
+    hide();
+    lower();	// lower the window below scrollbars
+    connect(parent, SIGNAL(selected(int)), SLOT(slotSelectionChanged()));
+    connect(parent, SIGNAL(collapsed(int)), SLOT(slotSelectionChanged()));
+    connect(parent, SIGNAL(expanded(int)), SLOT(slotSelectionChanged()));
+    connect(this, SIGNAL(done(int, const QString&)),
+	    parent, SIGNAL(editValueCommitted(int, const QString&)));
+}
+
+ValueEdit::~ValueEdit()
+{
+}
+
+void ValueEdit::terminate(bool commit)
+{
+    TRACE(commit?"ValueEdit::terminate(true)":"ValueEdit::terminate(false)");
+    if (!m_finished)
+    {
+	m_finished = true;
+	hide();	// will call focusOutEvent, that's why we need m_finished
+	if (commit) {
+	    emit done(m_row, text());
+	}
+    }
+}
+
+void ValueEdit::keyPressEvent(QKeyEvent *e)
+{
+    if(e->key() == Qt::Key_Return || e->key() == Qt::Key_Enter)
+	terminate(true);
+    else if(e->key() == Qt::Key_Escape)
+	terminate(false);
+    else
+	QLineEdit::keyPressEvent(e);
+}
+
+void ValueEdit::paintEvent(QPaintEvent* e)
+{
+    QLineEdit::paintEvent(e);
+
+    QPainter p(this);
+    p.drawRect(rect());
+}
+
+void ValueEdit::focusOutEvent(QFocusEvent* ev)
+{
+    TRACE("ValueEdit::focusOutEvent");
+    QFocusEvent* focusEv = static_cast<QFocusEvent*>(ev);
+    // Don't let a RMB close the editor
+    if (focusEv->reason() != QFocusEvent::Popup &&
+	focusEv->reason() != QFocusEvent::ActiveWindow)
+    {
+	terminate(true);
+    }
+}
+
+void ValueEdit::slotSelectionChanged()
+{   
+    TRACE("ValueEdit::slotSelectionChanged");
+    terminate(false);
+}
