@@ -1,4 +1,4 @@
-// $Id: gdbdriver.cpp,v 1.22.2.9 2004/05/19 20:19:58 jsixt Exp $
+// $Id: gdbdriver.cpp,v 1.46 2004/08/09 20:15:22 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -6,11 +6,8 @@
 #include "gdbdriver.h"
 #include "exprwnd.h"
 #include <qregexp.h>
-#if QT_VERSION >= 200
+#include <qstringlist.h>
 #include <klocale.h>			/* i18n */
-#else
-#include <kapp.h>
-#endif
 #include <ctype.h>
 #include <stdlib.h>			/* strtol, atoi */
 #include <string.h>			/* strcpy */
@@ -48,6 +45,10 @@ struct GdbCmdInfo {
     } argsNeeded;
 };
 
+#if 0
+// This is  how the QString data print statement generally looks like.
+// It is set by KDebugger via setPrintQStringDataCmd().
+
 static const char printQStringStructFmt[] =
 		// if the string data is junk, fail early
 		"print ($qstrunicode=($qstrdata=(%s))->unicode)?"
@@ -55,18 +56,10 @@ static const char printQStringStructFmt[] =
 		"(*(unsigned short*)$qstrunicode)@"
 		// limit the length
 		"(($qstrlen=(unsigned int)($qstrdata->len))>100?100:$qstrlen)"
-		// if unicode data is 0, check if it is QString::null
-		":($qstrdata==QString::null.d)\n";
-
-// However, gdb can't always find QString::null (I don't know why)
-// which results in an error; we autodetect this situation in
-// parseQt2QStrings and revert to a safe expression.
-// Same as above except for last line:
-static const char printQStringStructNoNullFmt[] =
-		"print ($qstrunicode=($qstrdata=(%s))->unicode)?"
-		"(*(unsigned short*)$qstrunicode)@"
-		"(($qstrlen=(unsigned int)($qstrdata->len))>100?100:$qstrlen)"
+		// if unicode data is 0, report a special value
 		":1==0\n";
+#endif
+static const char printQStringStructFmt[] = "print (0?\"%s\":$kdbgundef)\n";
 
 /*
  * The following array of commands must be sorted by the DC* values,
@@ -88,6 +81,7 @@ static GdbCmdInfo cmds[] = {
     { DCsetargs, "set args %s\n", GdbCmdInfo::argString },
     { DCsetenv, "set env %s %s\n", GdbCmdInfo::argString2 },
     { DCunsetenv, "unset env %s\n", GdbCmdInfo::argString },
+    { DCsetoption, "setoption %s %d\n", GdbCmdInfo::argStringNum},
     { DCcd, "cd %s\n", GdbCmdInfo::argString },
     { DCbt, "bt\n", GdbCmdInfo::argNone },
     { DCrun, "run\n", GdbCmdInfo::argNone },
@@ -109,6 +103,7 @@ static GdbCmdInfo cmds[] = {
     { DCenable, "enable %d\n", GdbCmdInfo::argNum },
     { DCdisable, "disable %d\n", GdbCmdInfo::argNum },
     { DCprint, "print %s\n", GdbCmdInfo::argString },
+    { DCprintDeref, "print *(%s)\n", GdbCmdInfo::argString },
     { DCprintStruct, "print %s\n", GdbCmdInfo::argString },
     { DCprintQStringStruct, printQStringStructFmt, GdbCmdInfo::argString},
     { DCframe, "frame %d\n", GdbCmdInfo::argNum },
@@ -118,7 +113,9 @@ static GdbCmdInfo cmds[] = {
     { DCinfothreads, "info threads\n", GdbCmdInfo::argNone },
     { DCinfobreak, "info breakpoints\n", GdbCmdInfo::argNone },
     { DCcondition, "condition %d %s\n", GdbCmdInfo::argNumString},
+    { DCsetpc, "set variable $pc=%s\n", GdbCmdInfo::argString },
     { DCignore, "ignore %d %d\n", GdbCmdInfo::argNum2},
+    { DCsetvariable, "set variable %s=%s\n", GdbCmdInfo::argString2 },
 };
 
 #define NUM_CMDS (int(sizeof(cmds)/sizeof(cmds[0])))
@@ -129,7 +126,7 @@ GdbDriver::GdbDriver() :
 	m_gdbMajor(4), m_gdbMinor(16)
 {
     strcpy(m_prompt, PROMPT);
-    m_promptLen = PROMPT_LEN;
+    m_promptMinLen = PROMPT_LEN;
     m_promptLastChar = PROMPT_LAST_CHAR;
 
 #ifndef NDEBUG
@@ -187,7 +184,6 @@ GdbDriver::GdbDriver() :
 	}
     }
     assert(strlen(printQStringStructFmt) <= MAX_FMTLEN);
-    assert(strlen(printQStringStructNoNullFmt) <= MAX_FMTLEN);
 #endif
 }
 
@@ -211,7 +207,17 @@ QString GdbDriver::defaultGdb()
 
 QString GdbDriver::defaultInvocation() const
 {
-    return defaultGdb();
+    if (m_defaultCmd.isEmpty()) {
+	return defaultGdb();
+    } else {
+	return m_defaultCmd;
+    }
+}
+
+QStringList GdbDriver::boolOptionList() const
+{
+    // no options
+    return QStringList();
 }
 
 bool GdbDriver::startup(QString cmdStr)
@@ -369,12 +375,6 @@ static void normalizeStringArg(QString& arg)
 }
 
 
-#if QT_VERSION < 200
-#define LATIN1(str) ((str).isNull() ? "" : (str).data())
-#else
-#define LATIN1(str) (str).latin1()
-#endif
-
 QString GdbDriver::makeCmdString(DbgCommand cmd, QString strArg)
 {
     assert(cmd >= 0 && cmd < NUM_CMDS);
@@ -387,14 +387,11 @@ QString GdbDriver::makeCmdString(DbgCommand cmd, QString strArg)
 	m_programWD = strArg;
     } else if (cmd == DCsetargs) {
 	// attach saved redirection
-#if QT_VERSION < 200
-	strArg.detach();
-#endif
 	strArg += m_redirect;
     }
 
     SIZED_QString(cmdString, MAX_FMTLEN+strArg.length());
-    cmdString.sprintf(cmds[cmd].fmt, LATIN1(strArg));
+    cmdString.sprintf(cmds[cmd].fmt, strArg.latin1());
     return cmdString;
 }
 
@@ -503,11 +500,11 @@ QString GdbDriver::makeCmdString(DbgCommand cmd, QString strArg, int intArg)
 	    if (slash >= 0)
 		strArg = strArg.right(strArg.length()-slash-1);
 	}
-	cmdString.sprintf(cmds[cmd].fmt, LATIN1(strArg), intArg);
+	cmdString.sprintf(cmds[cmd].fmt, strArg.latin1(), intArg);
     }
     else
     {
-	cmdString.sprintf(cmds[cmd].fmt, intArg, LATIN1(strArg));
+	cmdString.sprintf(cmds[cmd].fmt, intArg, strArg.latin1());
     }
     return cmdString;
 }
@@ -521,7 +518,7 @@ QString GdbDriver::makeCmdString(DbgCommand cmd, QString strArg1, QString strArg
     normalizeStringArg(strArg2);
 
     SIZED_QString(cmdString, MAX_FMTLEN+strArg1.length()+strArg2.length());
-    cmdString.sprintf(cmds[cmd].fmt, LATIN1(strArg1), LATIN1(strArg2));
+    cmdString.sprintf(cmds[cmd].fmt, strArg1.latin1(), strArg2.latin1());
     return cmdString;
 }
 
@@ -664,29 +661,6 @@ static bool parseErrorMessage(const char* output,
     return false;
 }
 
-#if QT_VERSION < 200
-struct QChar {
-    // this is the XChar2b on X11
-    uchar row;
-    uchar cell;
-    static QString toQString(QChar* unicode, int len);
-    QChar() : row(0), cell(0) { }
-    QChar(char c) : row(0), cell(c) { }
-    operator char() const { return row ? 0 : cell; }
-};
-
-QString QChar::toQString(QChar* unicode, int len)
-{
-    QString result(len+1);
-    char* data = result.data();
-    data[len] = '\0';
-    while (len >= 0) {
-	data[len] = unicode[len].cell;
-	--len;
-    }
-    return result;
-}
-#endif
 #if QT_VERSION >= 300
 union Qt2QChar {
     short s;
@@ -697,6 +671,15 @@ union Qt2QChar {
 };
 #endif
 
+void GdbDriver::setPrintQStringDataCmd(const char* cmd)
+{
+    // don't accept the command if it is empty
+    if (cmd == 0 || *cmd == '\0')
+	return;
+    assert(strlen(cmd) <= MAX_FMTLEN);
+    cmds[DCprintQStringStruct].fmt = cmd;
+}
+
 VarTree* GdbDriver::parseQCharArray(const char* output, bool wantErrorValue, bool qt3like)
 {
     VarTree* variable = 0;
@@ -709,23 +692,6 @@ VarTree* GdbDriver::parseQCharArray(const char* output, bool wantErrorValue, boo
     while (isspace(*p))
 	p++;
 
-    // check if this is an error indicating that gdb does not know about QString::null
-    if (cmds[DCprintQStringStruct].fmt == printQStringStructFmt &&
-	(strncmp(p, "Internal error: could not find static variable null", 51) == 0 ||
-	 /*
-	  * At least gdb 5.2.1 reports the following error when it accesses
-	  * QString::null. Checking for it can result in false positives
-	  * (where the error is actually triggered by a real NULL pointer
-	  * in a badly initialized QString variable) but the consequences
-	  * are mild.
-	  */
-	 strncmp(p, "Cannot access memory at address 0x0", 35) == 0))
-    {
-	/* QString::null doesn't work, use an alternate expression */
-	cmds[DCprintQStringStruct].fmt = printQStringStructNoNullFmt;
-	// continue and let parseOffErrorExpr catch the error
-    }
-
     // special case: empty string (0 repetitions)
     if (strncmp(p, "Invalid number 0 of repetitions", 31) == 0)
     {
@@ -810,19 +776,14 @@ VarTree* GdbDriver::parseQCharArray(const char* output, bool wantErrorValue, boo
 
 	    // escape a few frequently used characters
 	    char escapeCode = '\0';
-	    switch (char(ch)) {
+	    switch (ch.latin1()) {
 	    case '\n': escapeCode = 'n'; break;
 	    case '\r': escapeCode = 'r'; break;
 	    case '\t': escapeCode = 't'; break;
 	    case '\b': escapeCode = 'b'; break;
 	    case '\"': escapeCode = '\"'; break;
 	    case '\\': escapeCode = '\\'; break;
-#if QT_VERSION < 200
-		// since we only deal with ascii values must always escape '\0'
-	    case '\0': escapeCode = '0'; break;
-#else
 	    case '\0': if (value == 0) { escapeCode = '0'; } break;
-#endif
 	    }
 
 	    // add separator
@@ -932,12 +893,34 @@ static void skipNested(const char*& s, char opening, char closing)
 	}
 	p++;
     }
-    if (nest > 0) {
+    if (nest != 0) {
 	TRACE(QString().sprintf("parse error: mismatching %c%c at %-20.20s", opening, closing, s));
     }
     s = p;
 }
 
+/**
+ * Find the end of line that is not inside braces
+ */
+static void findEnd(const char*& s)
+{
+    const char* p = s;
+    while (*p && *p!='\n') {
+	while (*p && *p!='\n' && *p!='{')
+	    p++;
+	if (*p=='{') {
+	    p++;
+	    skipNested(p, '{', '}'); p--;
+	}
+    }
+    s = p;
+}
+
+static bool isNumberish(const char ch)
+{
+    return (ch>='0' && ch<='9') || ch=='.' || ch=='x'; 
+}
+
 void skipString(const char*& p)
 {
 moreStrings:
@@ -1107,6 +1090,7 @@ repeat:
 	//  (E *) 0xbffff450
 	//  red
 	//  &parseP (HTMLClueV *, char *)
+	//  Variable "x" is not available.
 
 	const char*p = s;
     
@@ -1197,6 +1181,13 @@ repeat:
 	    if (*p == '(') {
 		skipNested(p, '(', ')');
 	    }
+	} else if (strncmp(p, "Variable \"", 10) == 0) {
+	    // Variable "x" is not available.
+	    p += 10;		// skip to "
+	    skipName(p);
+	    if (strncmp(p, "\" is not available.", 19) == 0) {
+		p += 19;
+	    }
 	} else {
 	    // must be an enumeration value
 	    skipName(p);
@@ -1375,13 +1366,6 @@ static bool parseValueSeq(const char*& s, VarTree* variable)
     return true;
 }
 
-#if QT_VERSION < 200
-#define ISSPACE(c) isspace((c))
-#else
-// c is a QChar
-#define ISSPACE(c) (c).isSpace()
-#endif
-
 /**
  * Parses a stack frame.
  */
@@ -1490,19 +1474,19 @@ static void parseFrameInfo(const char*& s, QString& func,
      * simplify space that belongs to a string arguments that gdb sometimes
      * prints in the argument lists of the function.
      */
-    ASSERT(!ISSPACE(func[0]));		/* there must be non-white before first \n */
+    ASSERT(!isspace(func[0].latin1()));		/* there must be non-white before first \n */
     int nl = 0;
     while ((nl = func.find('\n', nl)) >= 0) {
 	// search back to the beginning of the whitespace
 	int startWhite = nl;
 	do {
 	    --startWhite;
-	} while (ISSPACE(func[startWhite]));
+	} while (isspace(func[startWhite].latin1()));
 	startWhite++;
 	// search forward to the end of the whitespace
 	do {
 	    nl++;
-	} while (ISSPACE(func[nl]));
+	} while (isspace(func[nl].latin1()));
 	// replace
 	func.replace(startWhite, nl-startWhite, " ");
 	/* continue searching for more \n's at this place: */
@@ -1510,7 +1494,6 @@ static void parseFrameInfo(const char*& s, QString& func,
     }
 }
 
-#undef ISSPACE
 
 /**
  * Parses a stack frame including its frame number
@@ -1777,8 +1760,13 @@ bool GdbDriver::parseThreadList(const char* output, QList<ThreadInfo>& threads)
     return true;
 }
 
+static bool parseNewBreakpoint(const char* o, int& id,
+			       QString& file, int& lineNo, QString& address);
+static bool parseNewWatchpoint(const char* o, int& id,
+			       QString& expr);
+
 bool GdbDriver::parseBreakpoint(const char* output, int& id,
-				QString& file, int& lineNo)
+				QString& file, int& lineNo, QString& address)
 {
     const char* o = output;
     // skip lines of that begin with "(Cannot find"
@@ -1789,20 +1777,41 @@ bool GdbDriver::parseBreakpoint(const char* output, int& id,
 	o++;				/* skip newline */
     }
 
-    if (strncmp(o, "Breakpoint ", 11) != 0)
-	return false;
-    
+    if (strncmp(o, "Breakpoint ", 11) == 0) {
+	output += 11;			/* skip "Breakpoint " */
+	return ::parseNewBreakpoint(output, id, file, lineNo, address);
+    } else if (strncmp(o, "Hardware watchpoint ", 20) == 0) {
+	output += 20;
+	return ::parseNewWatchpoint(output, id, address);
+    } else if (strncmp(o, "Watchpoint ", 11) == 0) {
+	output += 11;
+	return ::parseNewWatchpoint(output, id, address);
+    }
+    return false;
+}
+
+static bool parseNewBreakpoint(const char* o, int& id,
+			       QString& file, int& lineNo, QString& address)
+{
     // breakpoint id
-    output += 11;			/* skip "Breakpoint " */
     char* p;
-    int num = strtoul(output, &p, 10);
+    id = strtoul(o, &p, 10);
     if (p == o)
 	return false;
+
+    // check for the address
+    if (strncmp(p, " at 0x", 6) == 0) {
+	char* start = p+4;	       /* skip " at ", but not 0x */
+	p += 6;
+	while (isxdigit(*p))
+	    ++p;
+	address = FROM_LATIN1(start, p-start);
+    }
     
     // file name
     char* fileStart = strstr(p, "file ");
     if (fileStart == 0)
-	return false;
+	return !address.isEmpty();     /* parse error only if there's no address */
     fileStart += 5;
     
     // line number
@@ -1813,12 +1822,29 @@ bool GdbDriver::parseBreakpoint(const char* output, int& id,
     if (numStart == p)
 	return false;
 
-    id = num;
     file = fileName;
     lineNo = line-1;			/* zero-based! */
     return true;
 }
 
+static bool parseNewWatchpoint(const char* o, int& id,
+			       QString& expr)
+{
+    // watchpoint id
+    char* p;
+    id = strtoul(o, &p, 10);
+    if (p == o)
+	return false;
+
+    if (strncmp(p, ": ", 2) != 0)
+	return false;
+    p += 2;
+
+    // all the rest on the line is the expression
+    expr = FROM_LATIN1(p, strlen(p)).stripWhiteSpace();
+    return true;
+}
+
 void GdbDriver::parseLocals(const char* output, QList<VarTree>& newVars)
 {
     // check for possible error conditions
@@ -2090,6 +2116,41 @@ void GdbDriver::parseRegisters(const char* output, QList<RegisterInfo>& regs)
 		start = output;
 		skipNested(output, '{', '}');
 		value = FROM_LATIN1(start, output-start).simplifyWhiteSpace();
+	    } else {
+		// for gdb 5.3
+		// find first type that does not have an array, this is the RAW value
+		const char* end=start;
+		findEnd(end);
+		const char* cur=start; 
+		while (cur<end) {
+		    while (*cur != '=' && cur<end)
+			cur++;
+		    cur++; 
+		    while (isspace(*cur) && cur<end)
+			cur++;
+		    if (isNumberish(*cur)) {
+			end=cur;
+			while (*end && (*end!='}') && (*end!=',') && (*end!='\n'))
+			    end++;
+			QString rawValue = FROM_LATIN1(cur, end-cur).simplifyWhiteSpace();
+			reg->rawValue = rawValue;
+
+			if (rawValue.left(2)=="0x") {
+			    // ok we have a raw value, now get it's type
+			    end=cur-1;
+			    while (isspace(*end) || *end=='=') end--;
+			    end++;
+			    cur=end-1;
+			    while (*cur!='{' && *cur!=' ')
+				cur--;
+			    cur++;
+			    reg->type=FROM_LATIN1(cur, end-cur);
+			}
+
+			// end while loop 
+			cur=end;
+		    }
+		}
 		// skip to the end of line
 		while (*output != '\0' && *output != '\n')
 		    output++;
@@ -2111,28 +2172,33 @@ void GdbDriver::parseRegisters(const char* output, QList<RegisterInfo>& regs)
 	    value = FROM_LATIN1(start, output-start).simplifyWhiteSpace();
 
 	    /*
-	     * We split the raw from the cooked values. For this purpose,
-	     * we split off the first token (separated by whitespace). It
-	     * is the raw value. The remainder of the line is the cooked
-	     * value.
+	     * We split the raw from the cooked values.
+	     * Some modern gdbs explicitly say: "0.1234 (raw 0x3e4567...)".
+	     * Here, the cooked value comes first, and the raw value is in
+	     * the second part.
 	     */
-	    int pos = value.find(' ');
-	    if (pos < 0) {
-		reg->rawValue = value;
-		reg->cookedValue = QString();
-	    } else {
-		reg->rawValue = value.left(pos);
-		reg->cookedValue = value.mid(pos+1);
+	    int pos = value.find(" (raw ");
+	    if (pos >= 0)
+	    {
+		reg->cookedValue = value.left(pos);
+		reg->rawValue = value.mid(pos+6);
+		if (reg->rawValue.right(1) == ")")	// remove closing bracket
+		    reg->rawValue.truncate(reg->rawValue.length()-1);
+	    }
+	    else
+	    {
 		/*
-		 * Some modern gdbs explicitly say: "0.1234 (raw 0x3e4567...)".
-		 * Here the raw value is actually in the second part.
-		 */
-		if (reg->cookedValue.left(5) == "(raw ") {
-		    QString raw = reg->cookedValue.right(reg->cookedValue.length()-5);
-		    if (raw[raw.length()-1] == ')')	/* remove closing bracket */
-			raw = raw.left(raw.length()-1);
-		    reg->cookedValue = reg->rawValue;
-		    reg->rawValue = raw;
+		* In other cases we split off the first token (separated by
+		* whitespace). It is the raw value. The remainder of the line
+		* is the cooked value.
+		*/
+		int pos = value.find(' ');
+		if (pos < 0) {
+		    reg->rawValue = value;
+		    reg->cookedValue = QString();
+		} else {
+		    reg->rawValue = value.left(pos);
+		    reg->cookedValue = value.mid(pos+1);
 		}
 	    }
 	}
@@ -2145,19 +2211,22 @@ void GdbDriver::parseRegisters(const char* output, QList<RegisterInfo>& regs)
 
 bool GdbDriver::parseInfoLine(const char* output, QString& addrFrom, QString& addrTo)
 {
-    const char* start = strstr(output, "starts at address ");
+    // "is at address" or "starts at address"
+    const char* start = strstr(output, "s at address ");
     if (start == 0)
 	return false;
 
-    start += 18;
+    start += 13;
     const char* p = start;
     while (*p != '\0' && !isspace(*p))
 	p++;
     addrFrom = FROM_LATIN1(start, p-start);
 
     start = strstr(p, "and ends at ");
-    if (start == 0)
-	return false;
+    if (start == 0) {
+	addrTo = addrFrom;
+	return true;
+    }
 
     start += 12;
     p = start;
@@ -2188,7 +2257,7 @@ void GdbDriver::parseDisassemble(const char* output, QList<DisassembledCode>& co
     p++;
 
     // remove last line
-    const char* end = strstr(output, "\nEnd of assembler");
+    const char* end = strstr(output, "End of assembler");
     if (end == 0)
 	end = p + strlen(p);
 
@@ -2279,5 +2348,61 @@ QString GdbDriver::parseMemoryDump(const char* output, QList<MemoryDump>& memdum
     return QString();
 }
 
+QString GdbDriver::editableValue(VarTree* value)
+{
+    const char* s = value->m_value.latin1();
+
+    // if the variable is a pointer value that contains a cast,
+    // remove the cast
+    if (*s == '(') {
+	skipNested(s, '(', ')');
+	// skip space
+	while (isspace(*s))
+	    ++s;
+    }
+
+repeat:
+    const char* start = s;
+
+    if (strncmp(s, "0x", 2) == 0)
+    {
+	s += 2;
+	while (isxdigit(*s))
+	    ++s;
+
+	/*
+	 * What we saw so far might have been a reference. If so, edit the
+	 * referenced value. Otherwise, edit the pointer.
+	 */
+	if (*s == ':') {
+	    // a reference
+	    ++s;
+	    goto repeat;
+	}
+	// a pointer
+	// if it's a pointer to a string, remove the string
+	const char* end = s;
+	while (isspace(*s))
+	    ++s;
+	if (*s == '"') {
+	    // a string
+	    return FROM_LATIN1(start, end-start);
+	} else {
+	    // other pointer
+	    return FROM_LATIN1(start, strlen(start));
+	}
+    }
+
+    // else leave it unchanged (or stripped of the reference preamble)
+    return s;
+}
+
+QString GdbDriver::parseSetVariable(const char* output)
+{
+    // if there is any output, it is an error message
+    QString msg = output;
+    return msg.stripWhiteSpace();
+}
+
 
 #include "gdbdriver.moc"
