@@ -1,4 +1,4 @@
-// $Id: gdbdriver.h,v 1.12.2.1 2002/01/01 17:01:51 jsixt Exp $
+// $Id: gdbdriver.h,v 1.19 2004/08/09 20:15:22 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -18,6 +18,8 @@ public:
     
     virtual QString driverName() const;
     virtual QString defaultInvocation() const;
+    virtual QStringList boolOptionList() const;
+    void setDefaultInvocation(QString cmd) { m_defaultCmd = cmd; }
     static QString defaultGdb();
     virtual bool startup(QString cmdStr);
     virtual void commandFinished(CmdQueueItem* cmd);
@@ -47,6 +49,7 @@ public:
     virtual void terminate();
     virtual void detachAndTerminate();
     virtual void interruptInferior();
+    virtual void setPrintQStringDataCmd(const char* cmd);
     virtual VarTree* parseQCharArray(const char* output, bool wantErrorValue, bool qt3like);
     virtual void parseBackTrace(const char* output, QList<StackFrame>& stack);
     virtual bool parseFrameChange(const char* output, int& frameNo,
@@ -54,7 +57,7 @@ public:
     virtual bool parseBreakList(const char* output, QList<Breakpoint>& brks);
     virtual bool parseThreadList(const char* output, QList<ThreadInfo>& threads);
     virtual bool parseBreakpoint(const char* output, int& id,
-				 QString& file, int& lineNo);
+				 QString& file, int& lineNo, QString& address);
     virtual void parseLocals(const char* output, QList<VarTree>& newVars);
     virtual bool parsePrintExpr(const char* output, bool wantErrorValue,
 				VarTree*& var);
@@ -69,11 +72,14 @@ public:
 			       QString& addrFrom, QString& addrTo);
     virtual void parseDisassemble(const char* output, QList<DisassembledCode>& code);
     virtual QString parseMemoryDump(const char* output, QList<MemoryDump>& memdump);
+    virtual QString parseSetVariable(const char* output);
+    virtual QString editableValue(VarTree* value);
 protected:
     int m_gdbMajor, m_gdbMinor;
     QString m_programWD;		/* just an intermediate storage */
     QString m_redirect;			/* redirection to /dev/null */
     bool m_haveCoreFile;
+    QString m_defaultCmd;		/* how to invoke gdb */
 
     QString makeCmdString(DbgCommand cmd, QString strArg);
     QString makeCmdString(DbgCommand cmd, int intArg);
