@@ -1,5 +1,5 @@
 /*
- * $Id: ktreeview.h,v 1.10.2.3 2003/09/07 18:36:22 jsixt Exp $
+ * $Id: ktreeview.h,v 1.14 2003/09/07 18:35:02 jsixt Exp $
  * 
  * KTreeView class interface
  * 
@@ -36,10 +36,7 @@ typedef QStack<QString> KPath;
 class KTreeView;			/* forward declaration */
 
 /** Items for the KTreeView widget */
-class KTreeViewItem
-#if QT_VERSION >= 200
-: public Qt
-#endif
+class KTreeViewItem : public Qt
 {
     friend class KTreeView;
 public:
