@@ -1,22 +1,17 @@
-// $Id: main.cpp,v 1.21.2.5 2003/06/09 19:59:03 jsixt Exp $
+// $Id: main.cpp,v 1.35 2005/01/05 21:06:10 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
 
 
 #include <kapp.h>
-#if QT_VERSION >= 200
 #include <klocale.h>			/* i18n */
 #include <kmessagebox.h>
 #include <kglobal.h>
 #include <kstddirs.h>
 #include <kcmdlineargs.h> 
 #include <kaboutdata.h>
-#include <khelpmenu.h>
 #include <kpopupmenu.h>
-#else // QT_VERSION < 200
-#include <kmsgbox.h>
-#endif
 #include <kmenubar.h>
 #include "dbgmainwnd.h"
 #include "typetable.h"
@@ -38,31 +33,29 @@
 
 int main(int argc, char** argv)
 {
-#if QT_VERSION < 200
-    KApplication app(argc, argv, "kdbg");
-
-    QString about = "KDbg " VERSION " - ";
-    about += i18n("A Debugger\n"
-		  "by Johannes Sixt <Johannes.Sixt@telecom.at>\n"
-		  "with the help of many others");
-#else
     KAboutData aboutData("kdbg", "KDbg",
 			 VERSION,
 			 I18N_NOOP("A Debugger"),
 			 KAboutData::License_GPL, 
-			 "(c) 1998-2003 Johannes Sixt",
+			 "(c) 1998-2005 Johannes Sixt",
 			 0,		/* any text */
 			 "http://members.nextra.at/johsixt/kdbg.html",
 			 "Johannes.Sixt@telecom.at");
     aboutData.addAuthor("Johannes Sixt", 0, "Johannes.Sixt@telecom.at");
-    aboutData.addCredit("Max Judin",
-			I18N_NOOP("Docking windows"),
-			"novaprint@mtu-net.ru");
+    aboutData.addCredit("Keith Isdale",
+			I18N_NOOP("XSLT debugging"),
+			"k_isdale@tpg.com.au");
+    aboutData.addCredit("Daniel Kristjansson",
+			I18N_NOOP("Register groups and formating"),
+			"danielk@cat.nyu.edu");
     KCmdLineArgs::init( argc, argv, &aboutData );
 
     static KCmdLineOptions options[] = {
 	{ "t <file>", I18N_NOOP("transcript of conversation with the debugger"), 0 },
 	{ "r <device>", I18N_NOOP("remote debugging via <device>"), 0 },
+	{ "l <language>", I18N_NOOP("specify language: C, XSLT"), ""},
+	{ "x", I18N_NOOP("use language XSLT (deprecated)"), 0 },
+	{ "p <pid>", I18N_NOOP("specify PID of process to debug"), 0},
 	{ "+[program]", I18N_NOOP("path of executable to debug"), 0 },
 	{ "+[core]", I18N_NOOP("a core file to use"), 0},
 	{ 0, 0, 0 }
@@ -72,21 +65,9 @@ int main(int argc, char** argv)
     KApplication app;
 
     KGlobal::dirs()->addResourceType("types", "share/apps/kdbg/types");
-#endif
 
     DebuggerMainWnd debugger("kdbg_main");
 
-    // insert help menu
-    QPopupMenu* helpMenu;
-#if QT_VERSION < 200
-    helpMenu = kapp->getHelpMenu(false, about);
-#else
-    KHelpMenu* khm = new KHelpMenu(&debugger, &aboutData, false);
-    helpMenu = khm->menu();
-#endif
-    debugger.menuBar()->insertSeparator();
-    debugger.menuBar()->insertItem(i18n("&Help"), helpMenu);
-
     /* type libraries */
     TypeTable::initTypeLibraries();
 
@@ -105,70 +86,48 @@ int main(int argc, char** argv)
 
     // handle options
 
-#if QT_VERSION < 200
-    extern char *optarg;
-    extern int optind;
-    int ch;
-
-    const char* transcript = 0;
-    while ((ch = getopt(argc, argv, "r:t:")) != -1) {
-	switch (ch) {
-	case 'r':
-	    debugger.setRemoteDevice(optarg);
-	    break;
-	case 't':
-	    transcript = optarg;
-	    break;
-	default:
-	    TRACE(QString().sprintf("ignoring option -%c", ch));
-	}
-    }
-    argc -= optind - 1;
-    argv += optind - 1;
-#else
     KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
     QCString t = args->getOption("t");
     const char* transcript = t.data();
     QString remote = args->getOption("r");
     if (!remote.isEmpty())
 	debugger.setRemoteDevice(remote);
-#endif
+
+    QCString lang = args->getOption("l");
+
+    // deprecated option; overrides -l
+    if (args->isSet("x")){
+         /* run in xsldbg mode  */
+         lang = "xslt";
+    }
+
     // check environment variable for transcript file name
     if (transcript == 0) {
 	transcript = getenv("KDBG_TRANSCRIPT");
     }
     debugger.setTranscript(transcript);
 
-#if QT_VERSION < 200
-    if (!restored && argc > 1) {
-	// check for core file
-	if (argc > 2) {
-	    debugger.setCoreFile(argv[2]);
-	}
-	if (!debugger.debugProgram(argv[1])) {
-	    // failed
-	    TRACE("cannot start debugger");
-	    KMsgBox::message(&debugger, kapp->appName(),
-			     i18n("Cannot start debugger."),
-			     KMsgBox::STOP,
-			     i18n("OK"));
-	    debugger.setCoreFile("");
-	}
-    }
-#else
+    QString pid = args->getOption("p");
+
     if (!restored && args->count() > 0) {
-	// check for core file
-	if (args->count() > 1) {
+	// attach to process?
+	if (!pid.isEmpty()) {
+	    TRACE("pid: " + pid);
+	    debugger.setAttachPid(pid);
+	}
+	// check for core file; use it only if we're not attaching to a process
+	else if (args->count() > 1 && pid.isEmpty()) {
 	    debugger.setCoreFile(args->arg(1));
 	}
-	if (!debugger.debugProgram(args->arg(0))) {
+	if (!debugger.debugProgram(args->arg(0), lang)) {
 	    // failed
 	    TRACE("cannot start debugger");
 	    KMessageBox::error(&debugger, i18n("Cannot start debugger."));
-	    debugger.setCoreFile("");
+
+	    debugger.setCoreFile(QString());
+	    debugger.setAttachPid(QString());
 	}
     }
-#endif
 
     int rc = app.exec();
     return rc;
