@@ -1,4 +1,4 @@
-// $Id: mainwndbase.h,v 1.16.2.1 2002/01/31 20:14:03 jsixt Exp $
+// $Id: mainwndbase.h,v 1.26 2005/01/01 21:23:31 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -9,6 +9,7 @@
 #include <qlineedit.h>
 #include <qlayout.h>
 #include <qpushbutton.h>
+#include <qcstring.h>
 #include "exprwnd.h"
 #include "sys/types.h"			/* pid_t */
 
@@ -19,6 +20,7 @@ class UpdateUI;
 class KToolBar;
 class KStatusBar;
 class KProcess;
+class DebuggerDriver;
 
 class WatchWindow : public QWidget
 {
@@ -67,49 +69,37 @@ public:
      * Specifies the file where to write the transcript.
      */
     void setTranscript(const char* name);
+    /**
+     * Starts to debug the specified program using the specified language
+     * driver.
+     */
+    bool debugProgram(const QString& executable, QCString lang, QWidget* parent);
+    /**
+     * Specifies the process to attach to after the program is loaded.
+     */
+    void setAttachPid(const QString& pid);
 
     // the following are needed to handle program arguments
-    bool debugProgram(const QString& executable);
     void setCoreFile(const QString& corefile);
     void setRemoteDevice(const QString &remoteDevice);
-    /** returns true if the command was handled */
-    bool handleCommand(int item);
     /** helper around KFileDialog */
     static QString myGetFileName(QString caption,
 				 QString dir, QString filter,
 				 QWidget* parent);
     /** invokes the global options dialog */
-    virtual void doGlobalOptions();
-    /** add recent executable; moves it to top of list if present */
-    void addRecentExec(const QString& executable);
-    /** remove recent executable */
-    void removeRecentExec(const QString& executable);
-    /** start a new session; error popups appear on errors */
-    bool debugProgramInteractive(const QString& executable);
+    virtual void doGlobalOptions(QWidget* parent);
 
 protected:
     // settings
     virtual void saveSettings(KConfig*);
     virtual void restoreSettings(KConfig*);
 
-    // override must return the toolbar containing the animation and
-    // buttons to update
-    virtual KToolBar* dbgToolBar() = 0;
-    // override must return the statusbar
-    virtual KStatusBar* dbgStatusBar() = 0;
-    // override must return the main window (usually this)
-    virtual QWidget* dbgMainWnd() = 0;
     // override must return the integrated output window
     virtual TTYWindow* ttyWindow() = 0;
 
     // statusbar texts
     QString m_statusActive;
 
-    // animated button
-    QList<QPixmap> m_animation;
-    uint m_animationCounter;
-    void initAnimation();
-
     // output window
     QString m_outputTermCmdStr;
     QString m_outputTermKeepScript;
@@ -128,15 +118,19 @@ protected:
     QString m_sourceFilter;
     QString m_headerFilter;
 
-    // recent execs in File menu
-    QStrList m_recentExecList;
-
     // the debugger proper
     QString m_debuggerCmdStr;
     KDebugger* m_debugger;
-    void setupDebugger(ExprWnd* localVars,
+    void setupDebugger(QWidget* parent,
+		       ExprWnd* localVars,
 		       ExprWnd* watchVars,
 		       QListBox* backtrace);
+    DebuggerDriver* driverFromLang(QCString lang);
+    /**
+     * This function derives a driver name from the contents of the named
+     * file.
+     */
+    QCString driverNameFromFile(const QString& exe);
 
 public:
     /*
@@ -145,10 +139,7 @@ public:
      * virtual here since Qt signal mechanism fails miserably (because this
      * class will not be the left-most base class!).
      */
-    void updateUIItem(UpdateUI* item);
-    void updateLineItems();
-    void slotNewStatusMsg();
-    void slotAnimationTimeout();
+    void newStatusMsg(KStatusBar* statusbar);
     void slotDebuggerStarting();
 };
 
