@@ -1,4 +1,4 @@
-// $Id: pgmargs.cpp,v 1.16 2001/04/16 17:24:44 jsixt Exp $
+// $Id: pgmargs.cpp,v 1.20 2004/03/24 21:09:24 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -6,16 +6,19 @@
 #include "pgmargs.h"
 #include <kapp.h>
 #include <kfiledialog.h>
-#if QT_VERSION >= 200
 #include <klocale.h>			/* i18n */
-#endif
+#include "config.h"
 #include "mydebug.h"
 
-PgmArgs::PgmArgs(QWidget* parent, const QString& pgm, QDict<EnvVar>& envVars) :
+PgmArgs::PgmArgs(QWidget* parent, const QString& pgm, QDict<EnvVar>& envVars,
+		 const QStringList& allOptions) :
 	QDialog(parent, "pgmargs", true),
 	m_envVars(envVars),
 	m_label(this, "label"),
 	m_programArgs(this, "args"),
+	m_fileBrowse(this, "args_browse"),
+	m_optionsLabel(this, "options_label"),
+	m_options(this, "options"),
 	m_wdLabel(this, "wd_label"),
 	m_wd(this, "wd"),
 	m_wdBrowse(this, "wd_browse"),
@@ -29,22 +32,20 @@ PgmArgs::PgmArgs(QWidget* parent, const QString& pgm, QDict<EnvVar>& envVars) :
 	m_layout(this, 8),
 	m_edits(8),
 	m_buttons(8),
+	m_pgmArgsEdit(0),
 	m_wdEdit(0)
 {
     m_envVars.setAutoDelete(false);
 
-    QString title = kapp->getCaption();
+    QString title = kapp->caption();
     title += i18n(": Program arguments");
     setCaption(title);
 
-    QString fmt = i18n("Run %s with these arguments:");
-#if QT_VERSION < 200
-    QString lab(fmt.length() + pgm.length());
-#else
-    QString lab;
-#endif
-    lab.sprintf(fmt, pgm.data());
-    m_label.setText(lab);
+    QString lab = i18n("Run %1 with these arguments:");
+    {
+	QFileInfo fi = pgm;
+	m_label.setText(lab.arg(fi.fileName()));
+    }
     QSize s = m_label.sizeHint();
     /* don't make the label too wide if pgm name is very long */
     if (s.width() > 450)
@@ -55,10 +56,42 @@ PgmArgs::PgmArgs(QWidget* parent, const QString& pgm, QDict<EnvVar>& envVars) :
     m_programArgs.setMinimumSize(s);
     m_programArgs.setMaxLength(10000);
 
+    m_fileBrowse.setText("...");
+    s = m_fileBrowse.sizeHint();
+    m_fileBrowse.setMinimumSize(s);
+    connect(&m_fileBrowse, SIGNAL(clicked()), SLOT(browseArgs()));
+
+    int btnSpace = 0, numSpace = 0;    /* used to shift env buttons down */
+
+    // add options only if the option list is non-empty
+    if (!allOptions.isEmpty()) 
+    {
+	m_optionsLabel.setText(i18n("Options:"));
+	s = m_optionsLabel.sizeHint();
+	m_optionsLabel.setMinimumSize(s);
+	btnSpace += s.height();
+
+	m_options.setSelectionMode(QListBox::Multi);
+
+	// add 5 entries, then get size hint, then add the rest
+	QStringList rest = allOptions;
+	for (int i = 0; i < 5 && !rest.isEmpty(); i++) {
+	    m_options.insertItem(rest.first());
+	    rest.remove(rest.begin());
+	}
+	s = m_options.sizeHint();
+	m_options.insertStringList(rest);
+
+	m_options.setMinimumSize(s);
+	m_options.setMaximumHeight(s.height());
+	btnSpace += s.height();
+	numSpace += 2;
+    }
+
     m_wdLabel.setText(i18n("Working directory:"));
     s = m_wdLabel.sizeHint();
     m_wdLabel.setMinimumSize(s);
-    int btnSpace = s.height();
+    btnSpace += s.height();
 
     m_wdBrowse.setText("...");
     s = m_wdBrowse.sizeHint();
@@ -81,7 +114,7 @@ PgmArgs::PgmArgs(QWidget* parent, const QString& pgm, QDict<EnvVar>& envVars) :
     m_envList.setMinimumSize(330, 40);
     m_envList.addColumn(i18n("Name"), 100);
     m_envList.addColumn(i18n("Value"), 260);
-    connect(&m_envList, SIGNAL(currentChanged(QListViewItem*)),
+    connect(&m_envList, SIGNAL(selectionChanged(QListViewItem*)),
 	    SLOT(envListCurrentChanged(QListViewItem*)));
     initEnvList();
 
@@ -109,7 +142,15 @@ PgmArgs::PgmArgs(QWidget* parent, const QString& pgm, QDict<EnvVar>& envVars) :
     m_layout.addLayout(&m_edits, 10);
     m_layout.addLayout(&m_buttons);
     m_edits.addWidget(&m_label);
-    m_edits.addWidget(&m_programArgs);
+    m_edits.addLayout(&m_pgmArgsEdit);
+    if (!allOptions.isEmpty()) {
+	m_edits.addWidget(&m_optionsLabel);
+	m_edits.addWidget(&m_options);
+    } else {
+	m_optionsLabel.hide();
+	m_options.hide();
+	m_options.setEnabled(false);
+    }
     m_edits.addWidget(&m_wdLabel);
     m_edits.addLayout(&m_wdEdit);
     m_edits.addWidget(&m_envLabel);
@@ -117,23 +158,47 @@ PgmArgs::PgmArgs(QWidget* parent, const QString& pgm, QDict<EnvVar>& envVars) :
     m_edits.addWidget(&m_envList, 10);
     m_buttons.addWidget(&m_buttonOK);
     m_buttons.addWidget(&m_buttonCancel);
-    m_buttons.addSpacing(btnSpace + 2*m_edits.defaultBorder());
+    m_buttons.addSpacing(btnSpace + numSpace*m_edits.defaultBorder());
     m_buttons.addWidget(&m_buttonModify);
     m_buttons.addWidget(&m_buttonDelete);
     m_buttons.addStretch(10);
+    m_pgmArgsEdit.addWidget(&m_programArgs, 10);
+    m_pgmArgsEdit.addWidget(&m_fileBrowse);
     m_wdEdit.addWidget(&m_wd, 10);
     m_wdEdit.addWidget(&m_wdBrowse);
 
-    m_layout.activate();
-
     m_programArgs.setFocus();
-    resize(300, 300);
 }
 
 PgmArgs::~PgmArgs()
 {
 }
 
+// initializes the selected options
+void PgmArgs::setOptions(const QStringList& selectedOptions)
+{
+    QStringList::ConstIterator it;
+    for (it = selectedOptions.begin(); it != selectedOptions.end(); ++it) {
+	for (uint i = 0; i < m_options.count(); i++) {
+	    if (m_options.text(i) == *it) {
+		m_options.setSelected(i, true);
+		break;
+	    }
+	}
+    }
+}
+
+// returns the selected options
+QStringList PgmArgs::options() const
+{
+    QStringList sel;
+    for (uint i = 0; i < m_options.count(); i++) {
+	if (m_options.isSelected(i))
+	    sel.append(m_options.text(i));
+    }
+    return sel;
+}
+
 void PgmArgs::modifyVar()
 {
     QString name, value;
@@ -165,32 +230,36 @@ void PgmArgs::modifyVar()
 	val->item = new QListViewItem(&m_envList, name, value);	// inserts itself
 	m_envVars.insert(name, val);
     }
+    m_envList.setSelected(val->item, true);
+    m_buttonDelete.setEnabled(true);
 }
 
+// delete the selected item
 void PgmArgs::deleteVar()
 {
-    QString name, value;
-    parseEnvInput(name, value);
+    QListViewItem* item = m_envList.selectedItem();
+    if (item == 0)
+	return;
+    QString name = item->text(0);
 
     // lookup the value in the dictionary
     EnvVar* val = m_envVars[name];
-    if (val == 0)
-	return;
-
-    // delete from list
-    QListViewItem* item = val->item;
-    val->item = 0;
-    delete item;
-    // if this is a new item, delete it completely, otherwise zombie-ize it
-    if (val->status == EnvVar::EVnew) {
-	m_envVars.remove(name);
-	delete val;
-    } else {
-	// mark value deleted
-	val->status = EnvVar::EVdeleted;
+    if (val != 0)
+    {
+	// delete from list
+	val->item = 0;
+	// if this is a new item, delete it completely, otherwise zombie-ize it
+	if (val->status == EnvVar::EVnew) {
+	    m_envVars.remove(name);
+	    delete val;
+	} else {
+	    // mark value deleted
+	    val->status = EnvVar::EVdeleted;
+	}
     }
-    // clear the input
-    m_envVar.setText("");
+    delete item;
+    // there is no selected item anymore
+    m_buttonDelete.setEnabled(false);
 }
 
 void PgmArgs::parseEnvInput(QString& name, QString& value)
@@ -200,11 +269,7 @@ void PgmArgs::parseEnvInput(QString& name, QString& value)
     int equalSign = input.find('=');
     if (equalSign >= 0) {
 	name = input.left(equalSign).stripWhiteSpace();
-#if QT_VERSION < 200
-	value = input.mid(equalSign+1,input.length());
-#else
 	value = input.mid(equalSign+1);
-#endif
     } else {
 	name = input.stripWhiteSpace();
 	value = QString();		/* value is empty */
@@ -221,10 +286,14 @@ void PgmArgs::initEnvList()
 	name = it.currentKey();
 	val->item = new QListViewItem(&m_envList, name, val->value);	// inserts itself
     }
+
+    m_envList.setAllColumnsShowFocus(true);
+    m_buttonDelete.setEnabled(m_envList.selectedItem() != 0);
 }
 
 void PgmArgs::envListCurrentChanged(QListViewItem* item)
 {
+    m_buttonDelete.setEnabled(item != 0);
     if (item == 0)
 	return;
 
@@ -248,14 +317,24 @@ void PgmArgs::accept()
 void PgmArgs::browseWd()
 {
     // browse for the working directory
-#if QT_VERSION < 200
-    QString newDir = KDirDialog::getDirectory(wd(), this);
-#else
     QString newDir = KFileDialog::getExistingDirectory(wd(), this);
-#endif
     if (!newDir.isEmpty()) {
 	setWd(newDir);
     }
 }
 
+void PgmArgs::browseArgs()
+{
+    QString caption = i18n("Select a file name to insert as program argument");
+
+    // use the selection as default
+    QString f = m_programArgs.markedText();
+    f = KFileDialog::getSaveFileName(f, QString::null,
+				     this, caption);
+    // don't clear the selection of no file was selected
+    if (!f.isEmpty()) {
+	m_programArgs.insert(f);
+    }
+}
+
 #include "pgmargs.moc"
