@@ -1,4 +1,4 @@
-// $Id: regwnd.h,v 1.6 2000/11/11 15:00:00 jsixt Exp $
+// $Id: regwnd.h,v 1.11 2003/03/23 21:07:18 jsixt Exp $
 
 // Copyright by Judin Max, Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -7,11 +7,14 @@
 #define REGWND_H
 
 #include <qlistview.h>
+#include <map>
 
 class QPopupMenu;
 class RegisterViewItem;
+class GroupingViewItem;
 struct RegisterInfo;
 
+
 class RegisterView : public QListView
 {
     Q_OBJECT
@@ -20,15 +23,18 @@ public:
     ~RegisterView();
 
 protected slots:
-    void doubleClicked(QListViewItem*);
     void rightButtonClicked(QListViewItem*, const QPoint&, int);
     void slotModeChange(int);
     void updateRegisters(QList<RegisterInfo>&);
 
 private:
-    QListViewItem* m_lastItem;
+    void paletteChange(const QPalette& oldPal);
+    void updateGroupVisibility();
+    GroupingViewItem* findMatchingGroup(const QString& regName);
+    GroupingViewItem* findGroup(const QString& groupName);
     QPopupMenu* m_modemenu;
-    int m_mode;
+    typedef std::map<QString,RegisterViewItem*> RegMap;
+    RegMap m_registers;
 
 friend class RegisterViewItem;
 };
