@@ -1,4 +1,4 @@
-// $Id: sourcewnd.cpp,v 1.7.2.5 2004/05/19 20:35:51 jsixt Exp $
+// $Id: sourcewnd.cpp,v 1.15 2004/08/05 19:37:17 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -12,11 +12,7 @@
 #include <qkeycode.h>
 #include <kapp.h>
 #include <kiconloader.h>
-#if QT_VERSION >= 200
 #include <kglobalsettings.h>
-#else
-#include <ctype.h>
-#endif
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -30,24 +26,14 @@ SourceWindow::SourceWindow(const char* fileName, QWidget* parent, const char* na
     setNumCols(3);
 
     // load pixmaps
-#if QT_VERSION < 200
-    KIconLoader* loader = kapp->getIconLoader();
-    m_pcinner = loader->loadIcon("pcinner.xpm");
-    m_pcup = loader->loadIcon("pcup.xpm");
-    m_brkena = loader->loadIcon("brkena.xpm");
-    m_brkdis = loader->loadIcon("brkdis.xpm");
-    m_brktmp = loader->loadIcon("brktmp.xpm");
-    m_brkcond = loader->loadIcon("brkcond.xpm");
-    setFont(kapp->fixedFont);
-#else
     m_pcinner = UserIcon("pcinner");
     m_pcup = UserIcon("pcup");
     m_brkena = UserIcon("brkena");
     m_brkdis = UserIcon("brkdis");
     m_brktmp = UserIcon("brktmp");
     m_brkcond = UserIcon("brkcond");
+    m_brkorph = UserIcon("brkorph");
     setFont(KGlobalSettings::fixedFont());
-#endif
 }
 
 SourceWindow::~SourceWindow()
@@ -244,6 +230,12 @@ void SourceWindow::paintCell(QPainter* p, int row, int col)
 	    if (y < 0) y = 0;
 	    p->drawPixmap(0,y,m_brkcond);
 	}
+	if (item & liBPorphan) {
+	    // orphaned breakpoint marker
+	    int y = (h - m_brkcond.height())/2;
+	    if (y < 0) y = 0;
+	    p->drawPixmap(0,y,m_brkorph);
+	}
 	if (item & liPC) {
 	    // program counter in innermost frame
 	    int y = (h - m_pcinner.height())/2;
@@ -313,6 +305,8 @@ void SourceWindow::updateLineItems(const KDebugger* dbg)
 		flags |= liBPtemporary;
 	    if (!bp->condition.isEmpty() || bp->ignoreCount != 0)
 		flags |= liBPconditional;
+	    if (bp->isOrphaned())
+		flags |= liBPorphan;
 	    // update if changed
 	    int row = lineToRow(i, bp->address);
 	    if ((m_lineItems[row] & liBPany) != flags) {
@@ -450,11 +444,10 @@ void SourceWindow::keyPressEvent(QKeyEvent* ev)
     KTextView::keyPressEvent(ev);
 }
 
-#if QT_VERSION < 200
-#define ISIDENT(c) (isalnum((c)) || (c) == '_')
-#else
-#define ISIDENT(c) ((c).isLetterOrNumber() || (c) == '_')
-#endif
+static inline bool isident(QChar c)
+{
+    return c.isLetterOrNumber() || c.latin1() == '_';
+}
 
 bool SourceWindow::wordAtPoint(const QPoint& p, QString& word, QRect& r)
 {
@@ -473,6 +466,7 @@ bool SourceWindow::wordAtPoint(const QPoint& p, QString& word, QRect& r)
 
     // get the bounding rect of the text
     QPainter painter(this);
+    setupPainter(&painter);
     const QString& text = m_texts[row];
     QRect bound =
 	painter.boundingRect(left+2, top, 0,0,
@@ -488,7 +482,7 @@ bool SourceWindow::wordAtPoint(const QPoint& p, QString& word, QRect& r)
      */
     uint start = 0;
     while (start < text.length()) {
-	while (start < text.length() && !ISIDENT(text[start]))
+	while (start < text.length() && !isident(text[start]))
 	    start++;
 	if (start >= text.length())
 	    return false;
@@ -505,7 +499,7 @@ bool SourceWindow::wordAtPoint(const QPoint& p, QString& word, QRect& r)
 	// a word starts now
 	int startWidth = bound.width();
 	uint end = start;
-	while (end < text.length() && ISIDENT(text[end]))
+	while (end < text.length() && isident(text[end]))
 	    end++;
 	bound =
 	    painter.boundingRect(left+2, top, 0,0,
@@ -527,11 +521,7 @@ bool SourceWindow::wordAtPoint(const QPoint& p, QString& word, QRect& r)
 
 void SourceWindow::paletteChange(const QPalette& oldPal)
 {
-#if QT_VERSION < 200
-    setFont(kapp->fixedFont);
-#else
     setFont(KGlobalSettings::fixedFont());
-#endif
     KTextView::paletteChange(oldPal);
 }
 
