@@ -23,7 +23,7 @@ class Cl
 public:
 	Cl(int r);
 	virtual ~Cl();
-	virtual int f(int x) const;
+	virtual int f(int x);
 };
 
 typedef void (*PtrFunc)(E*, char);
@@ -68,7 +68,7 @@ void f(E e[3], char c)
 	emptyBase eb;
 	emptyDerived ed;
 	emptyNested en;
-	int ea[0];
+	int ea[3];
 	emptyVBase evb;
 	emptyVDerived evd;
 	emptyVNested evn;
@@ -77,7 +77,10 @@ void f(E e[3], char c)
 void strtest(const char* t)
 {
     const char* s = t;
-    std::cout << s << std::endl;
+    const char*& s2 = s;
+    if (t == 0)
+	strtest(s2);
+    std::cout << s2 << std::endl;
 }
 
 
@@ -150,7 +153,7 @@ Cl::~Cl()
 {
 }
 
-int Cl::f(int x) const
+int Cl::f(int x)
 {
 	int y = 2*x;
 	return y;
