@@ -1,4 +1,4 @@
-// $Id: ttywnd.cpp,v 1.8.2.1 2002/08/31 19:51:00 jsixt Exp $
+// $Id: ttywnd.cpp,v 1.10 2002/08/31 19:51:30 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -6,12 +6,8 @@
 #include <qsocketnotifier.h>
 #include <qpopupmenu.h>
 #include "ttywnd.h"
-#if QT_VERSION >= 200
 #include <kglobalsettings.h>
 #include <klocale.h>
-#else
-#include <kapp.h>
-#endif
 
 #include "config.h"
 #ifdef HAVE_FCNTL_H
@@ -147,13 +143,8 @@ TTYWindow::TTYWindow(QWidget* parent, const char* name) :
 	KTextView(parent, name),
 	m_tty(0)
 {
-#if QT_VERSION < 200
-    setFont(kapp->fixedFont);
-    setFocusPolicy(StrongFocus);
-#else
     setFont(KGlobalSettings::fixedFont());
     setFocusPolicy(WheelFocus);
-#endif
     clear();
 
     // create a context menu
@@ -205,11 +196,7 @@ void TTYWindow::slotAppend(char* buffer, int count)
 	    ++len;
 	}
 	if (len > 0) {
-#if QT_VERSION < 200
-	    QString str(start, len+1);
-#else
 	    QString str = QString::fromLatin1(start, len);
-#endif
 	    // update last line
 	    str = m_texts[m_texts.size()-1] + str;
 	    replaceLine(m_texts.size()-1, str);
