@@ -1,17 +1,16 @@
-// $Id: typetable.cpp,v 1.8.2.2 2003/09/07 18:36:22 jsixt Exp $
+// $Id: typetable.cpp,v 1.13 2004/08/09 20:15:22 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
 
 #include <qdir.h>
 #include <qptrlist.h>
-#if QT_VERSION < 200
-#include <kapp.h>
-#else
 #include <kglobal.h>
 #include <kstddirs.h>
-#endif
 #include <ksimpleconfig.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include "typetable.h"
 #include "mydebug.h"
 
@@ -35,31 +34,16 @@ void TypeTable::loadTypeTables()
 {
     typeTablesInited = true;
 
-#if QT_VERSION < 200
-    QDir dir(KApplication::kde_datadir() + "/kdbg/types");
-    const QStrList* files = dir.entryList("*.kdbgtt");
-	
-    if (files == 0) {
-	TRACE("no type tables found");
-	return;
-    }
-#else
     const QStringList files = KGlobal::dirs()->findAllResources("types", "*.kdbgtt");
     
     if (files.isEmpty()) {
 	TRACE("no type tables found");
 	return;
     }
-#endif
 
     QString fileName;
-#if QT_VERSION < 200
-    for (QListIterator<char> it(*files); it != 0; ++it) {
-	fileName = dir.filePath(it.current());
-#else
     for (QValueListConstIterator<QString> p = files.begin(); p != files.end(); ++p) {
 	fileName = *p;
-#endif
 	TypeTable* newTable = new TypeTable;
 	newTable->loadFromFile(fileName);
 	typeTables.append(newTable);
@@ -67,7 +51,8 @@ void TypeTable::loadTypeTables()
 }
 
 
-TypeTable::TypeTable()
+TypeTable::TypeTable() :
+	m_printQStringDataCmd(0)
 {
     m_typeDict.setAutoDelete(true);
     // aliasDict keeps only pointers to items into typeDict
@@ -76,6 +61,7 @@ TypeTable::TypeTable()
 
 TypeTable::~TypeTable()
 {
+    delete[] m_printQStringDataCmd;
 }
 
 
@@ -83,6 +69,7 @@ static const char TypeTableGroup[] = "Type Table";
 static const char LibDisplayName[] = "LibDisplayName";
 static const char ShlibRE[] = "ShlibRE";
 static const char EnableBuiltin[] = "EnableBuiltin";
+static const char PrintQStringCmd[] = "PrintQStringCmd";
 static const char TypesEntryFmt[] = "Types%d";
 static const char DisplayEntry[] = "Display";
 static const char AliasEntry[] = "Alias";
@@ -114,6 +101,13 @@ void TypeTable::loadFromFile(const QString& fileName)
     m_shlibNameRE = QRegExp(cf.readEntry(ShlibRE));
     cf.readListEntry(EnableBuiltin, m_enabledBuiltins);
 
+    QString printQString = cf.readEntry(PrintQStringCmd);
+    const char* ascii = printQString.ascii();
+    if (ascii == 0)
+	ascii = "";
+    m_printQStringDataCmd = new char[strlen(ascii)+1];
+    strcpy(m_printQStringDataCmd, ascii);
+
     /*
      * Get the types. We search for entries of kind Types1, Types2, etc.
      * because a single entry Types could get rather long for large
@@ -234,7 +228,8 @@ TypeInfo::~TypeInfo()
 
 ProgramTypeTable::ProgramTypeTable() :
 	m_parseQt2QStrings(false),
-	m_QCharIsShort(false)
+	m_QCharIsShort(false),
+	m_printQStringDataCmd(0)
 {
     m_types.setAutoDelete(false);	/* paranoia */
     m_aliasDict.setAutoDelete(false);	/* paranoia */
@@ -259,6 +254,9 @@ void ProgramTypeTable::loadTypeTable(TypeTable* table)
     if (!m_QCharIsShort) {
 	m_QCharIsShort = table->isEnabledBuiltin("QCharIsShort");
     }
+    if (!m_printQStringDataCmd && *table->printQStringDataCmd()) {
+	m_printQStringDataCmd = table->printQStringDataCmd();
+    }
 }
 
 TypeInfo* ProgramTypeTable::lookup(const char* type)
@@ -276,10 +274,6 @@ void ProgramTypeTable::registerAlias(const QString& name, TypeInfo* type)
     m_aliasDict.insert(name, type);
 }
 
-#if QT_VERSION < 200
-typedef QListIterator<char> QStrListIterator;
-#endif
-
 void ProgramTypeTable::loadLibTypes(const QStrList& libs)
 {
     QStrListIterator it = libs;
