@@ -1,4 +1,4 @@
-// $Id: typetable.h,v 1.5.2.1 2002/01/01 17:01:51 jsixt Exp $
+// $Id: typetable.h,v 1.7 2004/08/09 20:15:22 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -84,6 +84,11 @@ public:
      * Is the specified builtin feature enabled in this type library?
      */
     bool isEnabledBuiltin(const char* feature);
+
+    /**
+     * Returns the command to print the QString data.
+     */
+    const char* printQStringDataCmd() const { return m_printQStringDataCmd; }
     
 protected:
     /**
@@ -97,6 +102,7 @@ protected:
     QString m_displayName;
     QRegExp m_shlibNameRE;
     QStrList m_enabledBuiltins;
+    char* m_printQStringDataCmd;
 };
 
 
@@ -155,9 +161,15 @@ public:
      */
     bool qCharIsShort() const { return m_QCharIsShort; }
 
+    /**
+     * Returns the command to print the QString data.
+     */
+    const char* printQStringDataCmd() const { return m_printQStringDataCmd; }
+
 protected:
     QDict<TypeInfo> m_types;
     QDict<TypeInfo> m_aliasDict;
     bool m_parseQt2QStrings;
     bool m_QCharIsShort;
+    const char* m_printQStringDataCmd;
 };
