@@ -1,20 +1,15 @@
-// $Id: winstack.cpp,v 1.41.2.2 2002/03/24 20:50:20 jsixt Exp $
+// $Id: winstack.cpp,v 1.47 2005/01/01 21:21:59 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
 
 #include "winstack.h"
-#include "commandids.h"
 #include "sourcewnd.h"
 #include <qbrush.h>
 #include <qfileinfo.h>
 #include <qlistbox.h>
 #include <kapp.h>
-#if QT_VERSION >= 200
 #include <klocale.h>			/* i18n */
-#else
-#include <ctype.h>
-#endif
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -26,51 +21,23 @@ WinStack::WinStack(QWidget* parent, const char* name) :
 	QWidget(parent, name),
 	m_activeWindow(0),
 	m_windowMenu(0),
-	m_itemMore(0),
 	m_pcLine(-1),
 	m_valueTip(this),
 	m_tipLocation(1,1,10,10),
 	m_tabWidth(0)
 {
-    // Call menu implementation helper
-    initMenu();
-
     connect(&m_findDlg.m_buttonForward,
 	    SIGNAL(clicked()), SLOT(slotFindForward()));
     connect(&m_findDlg.m_buttonBackward,
 	    SIGNAL(clicked()), SLOT(slotFindBackward()));
 
     connect(this, SIGNAL(setTabWidth(int)), this, SLOT(slotSetTabWidth(int)));
-
-    // Check for right click event.
-    connect(this, SIGNAL(clickedRight(const QPoint &)),
-	    SLOT(slotWidgetRightClick(const QPoint &)));
 }
 
 WinStack::~WinStack()
 {
 }
 
-// All menu initializations.
-void WinStack::initMenu()
-{
-    // Init float popup menu.
-    m_menuFloat.insertItem(i18n("&Open Source..."), ID_FILE_OPEN);
-    m_menuFloat.insertSeparator();
-    m_menuFloat.insertItem(i18n("Step &into"), ID_PROGRAM_STEP);
-    m_menuFloat.insertItem(i18n("Step &over"), ID_PROGRAM_NEXT);
-    m_menuFloat.insertItem(i18n("Step o&ut"), ID_PROGRAM_FINISH);
-    m_menuFloat.insertItem(i18n("Run to &cursor"), ID_PROGRAM_UNTIL);
-    m_menuFloat.insertSeparator();
-    m_menuFloat.insertItem(i18n("Set/Clear &breakpoint"), ID_BRKPT_SET);
-
-    // Init float file popup.
-    m_menuFileFloat.insertItem(i18n("&Open Source..."), ID_FILE_OPEN);
-    m_menuFileFloat.insertSeparator();
-    m_menuFileFloat.insertItem(i18n("&Executable..."), ID_FILE_EXECUTABLE);
-    m_menuFileFloat.insertItem(i18n("&Core dump..."), ID_FILE_COREFILE);
-}
-
 void WinStack::setWindowMenu(QPopupMenu* menu)
 {
     m_windowMenu = menu;
@@ -78,58 +45,8 @@ void WinStack::setWindowMenu(QPopupMenu* menu)
 	return;
     }
     
-    // find entry More...
-    m_itemMore = menu->indexOf(ID_WINDOW_MORE);
-    // must contain item More...
-    ASSERT(m_itemMore >= 0);
-
-    m_textMore = menu->text(ID_WINDOW_MORE);
-    menu->removeItemAt(m_itemMore);
-}
-
-void WinStack::menuCallback(int item)
-{
-    TRACE("menu item=" + QString().setNum(item));
-    // check for window
-    if ((item & ~ID_WINDOW_INDEX_MASK) == ID_WINDOW_MORE) {
-	selectWindow(item & ID_WINDOW_INDEX_MASK);
-	return;
-    }
-    
-    switch (item) {
-    case ID_FILE_RELOAD:
-	if (m_activeWindow != 0) {
-	    TRACE("reloading one file");
-	    m_activeWindow->reloadFile();
-	}
-	break;
-    case ID_VIEW_FINDDLG:
-	if (m_findDlg.isVisible()) {
-	    m_findDlg.done(0);
-	} else {
-	    m_findDlg.show();
-	}
-	break;
-    case ID_BRKPT_SET:
-    case ID_BRKPT_TEMP:
-	{
-	    QString file;
-	    int lineNo;
-	    DbgAddr address;
-	    if (activeLine(file, lineNo, address))
-		emit toggleBreak(file, lineNo, address, item == ID_BRKPT_TEMP);
-	}
-	break;
-    case ID_BRKPT_ENABLE:
-	{
-	    QString file;
-	    int lineNo;
-	    DbgAddr address;
-	    if (activeLine(file, lineNo, address))
-		emit enadisBreak(file, lineNo, address);
-	}
-	break;
-    }
+    // hook up for menu activations
+    connect(menu, SIGNAL(activated(int)), this, SLOT(selectWindow(int)));
 }
 
 void WinStack::mousePressEvent(QMouseEvent* mouseEvent)
@@ -146,9 +63,8 @@ void WinStack::mousePressEvent(QMouseEvent* mouseEvent)
 
 void WinStack::reloadAllFiles()
 {
-    SourceWindow* fw;
-    for (fw = m_fileList.first(); fw != 0; fw = m_fileList.next()) {
-	fw->reloadFile();
+    for (int i = 0; i < m_fileList.size(); i++) {
+	m_fileList[i]->reloadFile();
     }
 }
 
@@ -182,9 +98,10 @@ void WinStack::activateFile(const QString& fileName)
 bool WinStack::activatePath(QString pathName, int lineNo, const DbgAddr& address)
 {
     // check whether the file is already open
-    SourceWindow* fw;
-    for (fw = m_fileList.first(); fw != 0; fw = m_fileList.next()) {
-	if (fw->fileName() == pathName) {
+    SourceWindow* fw = 0;
+    for (int i = 0; i < m_fileList.size(); i++) {
+	if (m_fileList[i]->fileName() == pathName) {
+	    fw = m_fileList[i];
 	    break;
 	}
     }
@@ -199,7 +116,7 @@ bool WinStack::activatePath(QString pathName, int lineNo, const DbgAddr& address
 	    return false;
 	}
 
-	m_fileList.insert(0, fw);
+	m_fileList.insertAt(0, fw);
 	connect(fw, SIGNAL(lineChanged()),SIGNAL(lineChanged()));
 	connect(fw, SIGNAL(clickedLeft(const QString&,int,const DbgAddr&,bool)),
 		SIGNAL(toggleBreak(const QString&,int,const DbgAddr&,bool)));
@@ -208,7 +125,7 @@ bool WinStack::activatePath(QString pathName, int lineNo, const DbgAddr& address
 
 	// Comunication when right button is clicked.
 	connect(fw, SIGNAL(clickedRight(const QPoint &)),
-		SLOT(slotFileWindowRightClick(const QPoint &)));
+		SIGNAL(filesRightClick(const QPoint &)));
 
 	// disassemble code
 	connect(fw, SIGNAL(disassemble(const QString&, int)),
@@ -233,7 +150,10 @@ bool WinStack::activatePath(QString pathName, int lineNo, const DbgAddr& address
 
 bool WinStack::activateWindow(SourceWindow* fw, int lineNo, const DbgAddr& address)
 {
-    int index = m_fileList.findRef(fw);
+    // lookup fw
+    int index = m_fileList.size()-1;
+    while (index >= 0 && m_fileList[index] != fw)
+	--index;
     ASSERT(index >= 0);
     if (index < 0) {
 	return false;
@@ -243,8 +163,8 @@ bool WinStack::activateWindow(SourceWindow* fw, int lineNo, const DbgAddr& addre
      * window menu, move it to the first position.
      */
     if (index >= 9) {
-	m_fileList.remove();
-	m_fileList.insert(0, fw);
+	m_fileList.removeAt(index);
+	m_fileList.insertAt(0, fw);
 	changeWindowMenu();
     }
 
@@ -260,11 +180,7 @@ bool WinStack::activateWindow(SourceWindow* fw, int lineNo, const DbgAddr& addre
 
     // set the focus to the new active window
     QWidget* oldActive = m_activeWindow;
-#if QT_VERSION < 200
-    fw->setFocusPolicy(QWidget::StrongFocus);
-#else
     fw->setFocusPolicy(QWidget::WheelFocus);
-#endif
     m_activeWindow = fw;
     if (oldActive != 0 && oldActive != fw) {
 	// disable focus on non-active windows
@@ -300,32 +216,34 @@ void WinStack::changeWindowMenu()
 	return;
     }
 
-    // delete window entries
-    while ((m_windowMenu->idAt(m_itemMore) & ~ID_WINDOW_INDEX_MASK) == ID_WINDOW_MORE) {
-	m_windowMenu->removeItemAt(m_itemMore);
+    const int N = 9;
+    // Delete entries at most the N window entries that we insert below.
+    // When we get here if the More entry was selected, we must make sure
+    // that we don't delete it (or we crash).
+    bool haveMore = m_windowMenu->count() > uint(N);
+    int k = haveMore ? N : m_windowMenu->count();
+
+    for (int i = 0; i < k; i++) {
+	m_windowMenu->removeItemAt(0);
     }
 
     // insert current windows
     QString text;
-    int index = 1;
-    SourceWindow* fw = 0;
-    for (fw = m_fileList.first(); fw != 0 && index < 10; fw = m_fileList.next()) {
-	text.sprintf("&%d ", index);
-	text += fw->fileName();
-	m_windowMenu->insertItem(text, ID_WINDOW_MORE+index, m_itemMore+index-1);
-	index++;
+    for (int i = 0; i < m_fileList.size() && i < N; i++) {
+	text.sprintf("&%d ", i+1);
+	text += m_fileList[i]->fileName();
+	m_windowMenu->insertItem(text, WindowMore+i+1, i);
     }
-    if (fw != 0) {
-	// there are still windows
-	m_windowMenu->insertItem(m_textMore, ID_WINDOW_MORE, m_itemMore+9);
+    // add More entry if we have none yet
+    if (!haveMore && m_fileList.size() > N) {
+	m_windowMenu->insertItem(i18n("&More..."), WindowMore, N);
     }
 }
 
 void WinStack::updateLineItems(const KDebugger* dbg)
 {
-    SourceWindow* fw = 0;
-    for (fw = m_fileList.first(); fw != 0; fw = m_fileList.next()) {
-	fw->updateLineItems(dbg);
+    for (int i = 0; i < m_fileList.size(); i++) {
+	m_fileList[i]->updateLineItems(dbg);
     }
 }
 
@@ -349,10 +267,9 @@ void WinStack::setPC(bool set, const QString& fileName, int lineNo,
     TRACE((set ? "set PC: " : "clear PC: ") + fileName +
 	  QString().sprintf(":%d#%d ", lineNo, frameNo) + address.asString());
     // find file
-    SourceWindow* fw = 0;
-    for (fw = m_fileList.first(); fw != 0; fw = m_fileList.next()) {
-	if (fw->fileNameMatches(fileName)) {
-	    fw->setPC(set, lineNo, address, frameNo);
+    for (int i = 0; i < m_fileList.size(); i++) {
+	if (m_fileList[i]->fileNameMatches(fileName)) {
+	    m_fileList[i]->setPC(set, lineNo, address, frameNo);
 	    break;
 	}
     }
@@ -360,7 +277,6 @@ void WinStack::setPC(bool set, const QString& fileName, int lineNo,
 
 void WinStack::resizeEvent(QResizeEvent*)
 {
-    ASSERT(m_activeWindow == 0 || m_fileList.findRef(m_activeWindow) >= 0);
     if (m_activeWindow != 0) {
 	m_activeWindow->resize(width(), height());
     }
@@ -380,30 +296,6 @@ void WinStack::slotFindBackward()
 			     SourceWindow::findBackward);
 }
 
-void WinStack::slotFileWindowRightClick(const QPoint & pos)
-{
-    if (m_menuFloat.isVisible())
-    {
-	m_menuFloat.hide();
-    }
-    else
-    {
-	m_menuFloat.popup(mapToGlobal(pos));
-    }
-}
-
-void WinStack::slotWidgetRightClick(const QPoint & pos)
-{
-    if (m_menuFileFloat.isVisible())
-    {
-	m_menuFileFloat.hide();
-    }
-    else
-    {
-	m_menuFileFloat.popup(mapToGlobal(pos));
-    }
-}
-
 void WinStack::maybeTip(const QPoint& p)
 {
     if (m_activeWindow == 0)
@@ -434,9 +326,10 @@ void WinStack::slotDisassembled(const QString& fileName, int lineNo,
 				const QList<DisassembledCode>& disass)
 {
     // lookup the file
-    SourceWindow* fw;
-    for (fw = m_fileList.first(); fw != 0; fw = m_fileList.next()) {
-	if (fw->fileNameMatches(fileName)) {
+    SourceWindow* fw = 0;
+    for (int i = 0; i < m_fileList.size(); i++) {
+	if (m_fileList[i]->fileNameMatches(fileName)) {
+	    fw = m_fileList[i];
 	    break;
 	}
     }
@@ -467,6 +360,59 @@ void WinStack::slotSetTabWidth(int numChars)
     m_tabWidth = numChars;
 }
 
+void WinStack::slotFileReload()
+{
+    if (m_activeWindow != 0) {
+	TRACE("reloading one file");
+	m_activeWindow->reloadFile();
+    }
+}
+
+void WinStack::slotViewFind()
+{
+    if (m_findDlg.isVisible()) {
+	m_findDlg.done(0);
+    } else {
+	m_findDlg.show();
+    }
+}
+
+void WinStack::slotBrkptSet()
+{
+    QString file;
+    int lineNo;
+    DbgAddr address;
+    if (activeLine(file, lineNo, address))
+	emit toggleBreak(file, lineNo, address, false);
+}
+
+void WinStack::slotBrkptSetTemp()
+{
+    QString file;
+    int lineNo;
+    DbgAddr address;
+    if (activeLine(file, lineNo, address))
+	emit toggleBreak(file, lineNo, address, true);
+}
+
+void WinStack::slotBrkptEnable()
+{
+    QString file;
+    int lineNo;
+    DbgAddr address;
+    if (activeLine(file, lineNo, address))
+	emit enadisBreak(file, lineNo, address);
+}
+
+void WinStack::slotMoveProgramCounter()
+{
+    QString file;
+    int lineNo;
+    DbgAddr address;
+    if (activeLine(file, lineNo, address))
+	emit moveProgramCounter(file, lineNo, address);
+}
+
 
 ValueTip::ValueTip(WinStack* parent) :
 	QToolTip(parent)
@@ -506,7 +452,7 @@ MoreWindowsDialog::MoreWindowsDialog(QWidget* parent) :
 	m_layout(this, 8),
 	m_buttons(4)
 {
-    QString title = kapp->getCaption();
+    QString title = kapp->caption();
     title += i18n(": Open Windows");
     setCaption(title);
 
@@ -542,31 +488,34 @@ MoreWindowsDialog::~MoreWindowsDialog()
 
 void WinStack::selectWindow(int id)
 {
+    // react only on menu entries concerning windows
+    if ((id & ~WindowMask) != WindowMore) {
+	return;
+    }
+
+    id &= WindowMask;
+
     int index = 0;
 
     if (id == 0) {
 	// more windows selected: show windows in a list
 	MoreWindowsDialog dlg(this);
-	int i = 0;
-	for (SourceWindow* fw = m_fileList.first(); fw != 0; fw = m_fileList.next()) {
-	    dlg.insertString(fw->fileName());
-	    if (m_activeWindow == fw) {
+	for (int i = 0; i < m_fileList.size(); i++)
+	{
+	    dlg.insertString(m_fileList[i]->fileName());
+	    if (m_activeWindow == m_fileList[i]) {
 		index = i;
 	    }
-	    i++;
 	}
 	dlg.setListIndex(index);
 	if (dlg.exec() == QDialog::Rejected)
 	    return;
 	index = dlg.listIndex();
     } else {
-	index = (id & ID_WINDOW_INDEX_MASK)-1;
+	index = (id & WindowMask)-1;
     }
 
-    SourceWindow* fw = m_fileList.first();
-    for (; index > 0; index--) {
-	fw = m_fileList.next();
-    }
+    SourceWindow* fw = m_fileList[index];
     ASSERT(fw != 0);
 	
     activateWindow(fw, -1, DbgAddr());
@@ -583,7 +532,7 @@ FindDialog::FindDialog() :
 	m_layout(this, 8),
 	m_buttons(4)
 {
-    setCaption(QString(kapp->getCaption()) + i18n(": Search"));
+    setCaption(QString(kapp->caption()) + i18n(": Search"));
 
     m_searchText.setMinimumSize(330, 24);
     m_searchText.setMaxLength(10000);
@@ -600,11 +549,9 @@ FindDialog::FindDialog() :
 
     // get maximum size of buttons
     QSize maxSize(80,30);
-#if QT_VERSION >= 140
     maxSize.expandedTo(m_buttonForward.sizeHint());
     maxSize.expandedTo(m_buttonBackward.sizeHint());
     maxSize.expandedTo(m_buttonClose.sizeHint());
-#endif
 
     m_buttonForward.setMinimumSize(maxSize);
     m_buttonBackward.setMinimumSize(maxSize);
